// Copyright 2002-2013, University of Colorado Boulder

define( function( require ) {
  'use strict';

  var inherit = require( 'PHET_CORE/inherit' );

  var Color = require( 'Color' );
  var MathUtil = require( 'MathUtil' );
  var Vector2D = require( 'Vector2D' );
  var LGColors = require( 'LGColors' );

//Class declaration for Line

  /**
   * Constructor for Line
   * @param {double} x1
   * @param {double} y1
   * @param {double} x2
   * @param {double} y2
   * @param {Color} color
   */
  function Line( x1, y1, x2, y2, color ) {
    assert( x1 != x2 || y1 != y2 );
    this.x1 = x1;
    this.y1 = y1;
    this.x2 = x2;
    this.y2 = y2;
    this.rise = y2 - y1;
    this.run = x2 - x1;
    this.color = color;
  }

  return inherit( Object, Line, {

    createPointSlope: function( x1, y1, rise, run, color ) {
      return new Line( x1, y1, x1 + run, y1 + rise, color );
    },


    createPointSlope: function( x1, y1, rise, run ) {
      return this.createPointSlope( x1, y1, rise, run, DEFAULT_COLOR );
    },


    createSlopeIntercept: function( rise, run, yIntercept, color ) {
      return this.createPointSlope( 0, yIntercept, rise, run, color );
    },


    createSlopeIntercept: function( rise, run, yIntercept ) {
      return this.createSlopeIntercept( rise, run, yIntercept, DEFAULT_COLOR );
    },


    withColor: function( color ) {
      return new Line( x1, y1, x2, y2, color );
    },


    getSlope: function() {
      if ( this.undefinedSlope() ) {
        return this.NaN;
      }
      else {
        return rise / run;
      }
    },


    undefinedSlope: function() {
      return run == 0;
    },


    solveY: function( x ) {
      if ( this.undefinedSlope() ) {
        return this.NaN;
      }
      else {
        return (this.getSlope() * (x - x1)) + y1;
      }
    },


    solveX: function( y ) {
      if ( rise == 0 || run == 0 ) {
        return this.NaN;
      }
      else {
        return ((y - y1) / (rise / run)) + x1;
      }
    },


    same: function( line ) {
      return (line != null) && this.onLine( this.x1, this.y1 ) && this.onLine( this.x2, this.y2 );
    },


    getSimplifiedRise: function() {
      if ( this.slopeIsSimplifiable() ) {
        return (rise / MathUtil.getGreatestCommonDivisor( rise, run ));
      }
      else {
        return rise;
      }
    },


    getSimplifiedRun: function() {
      if ( this.slopeIsSimplifiable() ) {
        return (run / MathUtil.getGreatestCommonDivisor( rise, run ));
      }
      else {
        return run;
      }
    },


    slopeIsSimplifiable: function() {
      return (rise != 0) && (run != 0) && MathUtil.isInteger( rise ) && MathUtil.isInteger( run );
    },


    onLine: function( p ) {
      return this.onLine( p.getX(), p.getY() );
    },


    onLine: function( x, y ) {
      if ( rise == 0 ) {
        return (y == y1);
      }
      else if ( run == 0 ) {
        return (x == x1);
      }
      else {
        return (x == this.solveX( y ));
      }
    },


    /**
     * Gets the the y-intercept as a simplified fraction.
     * This is valid only if (x1,y1) and (x2,y2) are at integer locations on the grid.
     */
    getYIntercept: function() {
      assert( MathUtil.isInteger( x1 ) && MathUtil.isInteger( y1 ) && MathUtil.isInteger( rise ) && MathUtil.isInteger( run ) );
      if ( rise == 0 || run == 0 ) {
        return new Fraction( y1, 1 );
      }
      var numerator = ((y1 * run) - (x1 * rise));
      var denominator = run;
      var gcd = MathUtil.getGreatestCommonDivisor( numerator, denominator );
      return new Fraction( numerator / gcd, denominator / gcd );
    },


    toString: function() {
      return "x1=" + x1 + ", y1=" + y1 + ", x2=" + x2 + ", y2=" + y2 + ", rise=" + rise + ", run=" + run + ", color=" + color;
    },


  } );

} );

