// Copyright 2002-2013, University of Colorado Boulder

define( function( require ) {
  'use strict';

  var inherit = require( 'PHET_CORE/inherit' );

  var Color = require( 'Color' );
  var ActionEvent = require( 'ActionEvent' );
  var ActionListener = require( 'ActionListener' );
  var Line2D = require( 'Line2D' );
  var Rectangle2D = require( 'Rectangle2D' );
  var Property = require( 'Property' );
  var ParameterSet = require( 'ParameterSet' );
  var ObservableList = require( 'ObservableList' );
  var VoidFunction0 = require( 'VoidFunction0' );
  var VoidFunction1 = require( 'VoidFunction1' );
  var PhetPNode = require( 'PhetPNode' );
  var ControlPanelNode = require( 'ControlPanelNode' );
  var TextButtonNode = require( 'TextButtonNode' );
  var ToggleButtonNode = require( 'ToggleButtonNode' );
  var ZeroOffsetNode = require( 'ZeroOffsetNode' );
  var LGColors = require( 'LGColors' );
  var LGConstants = require( 'LGConstants' );
  var Images = require( 'Images' );
  var Strings = require( 'Strings' );
  var ParameterKeys = require( 'ParameterKeys' );
  var UserComponents = require( 'UserComponents' );
  var Line = require( 'Line' );
  var PNode = require( 'PNode' );
  var PPath = require( 'PPath' );

//Class declaration for EquationControls

  /**
   * Constructor
   *
   * @param titleNode               title that will be placed next to the minimize/maximize button
   * @param interactiveLine         the line that can be manipulated by the user
   * @param savedLines              lines that have been saved by the user
   * @param maximized               is the control panel maximized (true) or minimized (false)?
   * @param linesVisible            are lines visible on the graph?
   * @param interactiveEquationNode node that implements the interactive equation
   */
  /**
   * Constructor for EquationControls
   * @param {PNode} titleNode
   * @param {Property<Line>} interactiveLine
   * @param {ObservableList<Line>} savedLines
   * @param {Property<Boolean>} maximized
   * @param {Property<Boolean>} linesVisible
   * @param {PNode} interactiveEquationNode
   */
  function EquationControls( titleNode, interactiveLine, savedLines, maximized, linesVisible, interactiveEquationNode ) {
    var minimizeMaximizeButtonNode = new ToggleButtonNode( this.equationMinimizeMaximizeButton, maximized, this.MINIMIZE_BUTTON, this.MAXIMIZE_BUTTON )
    {

      getParameterSet: function() {
        return super.getParameterSet().with( this.maximized, !maximized.get() );
      }
    }
    ;
    var equationNode = new ZeroOffsetNode( interactiveEquationNode );
    var saveLineButton = new TextButtonNode( this.SAVE_LINE, this.CONTROL_FONT, this.SAVE_LINE_BUTTON )
    {

      {
        this.setUserComponent( this.saveLineButton );
      }
    }
    ;
    var eraseLinesButton = new TextButtonNode( this.ERASE_LINES, this.CONTROL_FONT, this.ERASE_LINES_BUTTON )
    {

      {
        this.setUserComponent( this.eraseLinesButton );
        this.setEnabled( false );
      }
    }
    ;
    var panelNode = new PNode();
    panelNode.addChild( titleNode );
    panelNode.addChild( minimizeMaximizeButtonNode );
    var subPanelNode = new PNode();
    panelNode.addChild( subPanelNode );
    subPanelNode.addChild( equationNode );
    subPanelNode.addChild( saveLineButton );
    subPanelNode.addChild( eraseLinesButton );
    var separatorColor = new Color( 212, 212, 212 );
    var titleSeparator = new PPath( new Line2D.Double( 0, 0, 1, 0 ) )
    {

      {
        this.setStrokePaint( separatorColor );
      }
    }
    ;
    var buttonsSeparator = new PPath( new Line2D.Double( 0, 0, 1, 0 ) )
    {

      {
        this.setStrokePaint( separatorColor );
      }
    }
    ;
    subPanelNode.addChild( titleSeparator );
    subPanelNode.addChild( buttonsSeparator );
    var buttonsXSpacing = 10;
    {
      var ySpacing = 10;
      var titleHeight = Math.max( titleNode.getFullBoundsReference().getHeight(), minimizeMaximizeButtonNode.getFullBoundsReference().getHeight() );
      minimizeMaximizeButtonNode.setOffset( 0, (titleHeight - minimizeMaximizeButtonNode.getFullBoundsReference().getHeight()) / 2 );
      titleNode.setOffset( minimizeMaximizeButtonNode.getFullBoundsReference().getMaxX() + buttonsXSpacing, (titleHeight - titleNode.getFullBoundsReference().getHeight()) / 2 );
      titleSeparator.setOffset( 0, titleHeight + ySpacing );
      equationNode.setOffset( 0, titleSeparator.getFullBoundsReference().getMaxY() + ySpacing );
      buttonsSeparator.setOffset( 0, equationNode.getFullBoundsReference().getMaxY() + ySpacing );
      saveLineButton.setOffset( 0, buttonsSeparator.getFullBoundsReference().getMaxY() + ySpacing );
      eraseLinesButton.setOffset( saveLineButton.getFullBoundsReference().getMaxX() + buttonsXSpacing, saveLineButton.getYOffset() );
    }
    var panelWidth = panelNode.getFullBoundsReference().getWidth() + 5;
    var strutNode = new PPath( new Rectangle2D.Double( 0, 0, panelWidth, 1 ) );
    strutNode.setStroke( null );
    strutNode.setPickable( false );
    panelNode.addChild( strutNode );
    strutNode.moveToBack();
    titleSeparator.setPathTo( new Line2D.Double( 0, 0, panelWidth, 0 ) );
    buttonsSeparator.setPathTo( new Line2D.Double( 0, 0, panelWidth, 0 ) );
    {
      var centerX = panelWidth / 2;
      minimizeMaximizeButtonNode.setOffset( 5, minimizeMaximizeButtonNode.getYOffset() );
      titleNode.setOffset( centerX - (titleNode.getFullBoundsReference().getWidth() / 2), titleNode.getYOffset() );
      equationNode.setOffset( centerX - (equationNode.getFullBoundsReference().getWidth() / 2), equationNode.getYOffset() );
      saveLineButton.setOffset( centerX - saveLineButton.getFullBoundsReference().getWidth() - (buttonsXSpacing / 2), saveLineButton.getYOffset() );
      eraseLinesButton.setOffset( centerX + (buttonsXSpacing / 2), eraseLinesButton.getYOffset() );
    }
    this.addChild( new ControlPanelNode( panelNode, this.EQUATION_CONTROL_PANEL ) );
    maximized.addObserver( new VoidFunction1()
    {

      apply: function( maximized ) {
        if ( maximized ) {
          panelNode.addChild( subPanelNode );
        }
        else {
          panelNode.removeChild( subPanelNode );
        }
      }
    }
  )
    ;
    saveLineButton.addActionListener( new ActionListener()
    {

      actionPerformed: function( e ) {
        savedLines.add( interactiveLine.get().withColor( this.SAVED_LINE_NORMAL ) );
      }
    }
  )
    ;
    eraseLinesButton.addActionListener( new ActionListener()
    {

      actionPerformed: function( e ) {
        savedLines.clear();
      }
    }
  )
    ;
    var enableButtons = new VoidFunction0()
    {

      apply: function() {
        saveLineButton.setEnabled( linesVisible.get() );
        eraseLinesButton.setEnabled( linesVisible.get() && (savedLines.size() > 0) );
      }
    }
    ;
    var savedLinesChanged = new VoidFunction1()
    {

      apply: function( line ) {
        enableButtons.apply();
      }
    }
    ;
    savedLines.addElementAddedObserver( savedLinesChanged );
    savedLines.addElementRemovedObserver( savedLinesChanged );
    linesVisible.addObserver( new VoidFunction1()
    {

      apply: function( visible ) {
        enableButtons.apply();
      }
    }
  )
    ;
  }

  return inherit( PhetPNode, EquationControls, {

  } );

} );

