// Copyright 2002-2013, University of Colorado Boulder

define( function( require ) {
  'use strict';

  var inherit = require( 'PHET_CORE/inherit' );

  var Dimension = require( 'Dimension' );
  var Font = require( 'Font' );
  var Insets = require( 'Insets' );
  var MessageFormat = require( 'MessageFormat' );
  var JComponent = require( 'JComponent' );
  var JFrame = require( 'JFrame' );
  var JPanel = require( 'JPanel' );
  var WindowConstants = require( 'WindowConstants' );
  var Property = require( 'Property' );
  var SettableNot = require( 'SettableNot' );
  var ObservableList = require( 'ObservableList' );
  var VoidFunction1 = require( 'VoidFunction1' );
  var PropertyCheckBox = require( 'PropertyCheckBox' );
  var PropertyCheckBoxWithIcon = require( 'PropertyCheckBoxWithIcon' );
  var GridPanel = require( 'GridPanel' );
  var Anchor = require( 'Anchor' );
  var Fill = require( 'Fill' );
  var PhetFont = require( 'PhetFont' );
  var PhetPCanvas = require( 'PhetPCanvas' );
  var ControlPanelNode = require( 'ControlPanelNode' );
  var LGColors = require( 'LGColors' );
  var Strings = require( 'Strings' );
  var UserComponents = require( 'UserComponents' );
  var Line = require( 'Line' );
  var PNode = require( 'PNode' );
  var UserComponents = require( 'UserComponents' );

//Class declaration for GraphControls

  /**
   * Constructor for GraphControls
   * @param {Property<Boolean>} linesVisible
   * @param {Property<Boolean>} slopeVisible
   * @param {ObservableList<Line>} standardLines
   */
  function GraphControls( linesVisible, slopeVisible, standardLines ) {
    this( linesVisible, slopeVisible, standardLines, true );
  }

  /**
   * Constructor for GraphControls
   * @param {Property<Boolean>} linesVisible
   * @param {Property<Boolean>} slopeVisible
   */
  function GraphControls( linesVisible, slopeVisible ) {
    this( linesVisible, slopeVisible, new ObservableList(), false );
  }

  /**
   * Constructor
   *
   * @param linesVisible         are lines visible on the graph?
   * @param slopeVisible         are the slope (rise/run) brackets visible on the graphed line?
   * @param standardLines        standard lines (eg, y=x) that are available for viewing
   * @param includeStandardLines should check boxes for standard lines be accessible?
   */
  /**
   * Constructor for GraphControls
   * @param {Property<Boolean>} linesVisible
   * @param {Property<Boolean>} slopeVisible
   * @param {ObservableList<Line>} standardLines
   * @param {boolean} includeStandardLines
   */
  function GraphControls( linesVisible, slopeVisible, standardLines, includeStandardLines ) {
    var yEqualsXVisible = new Property( standardLines.contains( this.Y_EQUALS_X_LINE ) );
    var yEqualsNegativeXVisible = new Property( standardLines.contains( this.Y_EQUALS_NEGATIVE_X_LINE ) );
    var hideLinesCheckBox = new PropertyCheckBox( this.linesCheckBox, this.HIDE_LINES, new SettableNot( linesVisible ) );
    hideLinesCheckBox.setFont( CONTROL_FONT );
    var positiveCheckBox = new PropertyCheckBoxWithIcon( yEqualsXCheckBox, Y_EQUALS_X, CONTROL_FONT, GraphNode.createYEqualsXIcon( 60, this.Y_EQUALS_X ), yEqualsXVisible );
    var negativeCheckBox = new PropertyCheckBoxWithIcon( yEqualsNegativeXCheckBox, Y_EQUALS_NEGATIVE_X, CONTROL_FONT, GraphNode.createYEqualsNegativeXIcon( 60, this.Y_EQUALS_NEGATIVE_X ), yEqualsNegativeXVisible );
    var slopeCheckBox = new PropertyCheckBoxWithIcon( riseOverRunCheckBox, this.SLOPE, CONTROL_FONT, SlopeToolNode.createIcon( 60 ), slopeVisible );
    var panel = new GridPanel();
    panel.setGridX( 0 );
    panel.setInsets( new Insets( 2, 2, 2, 2 ) );
    panel.setAnchor( this.CENTER );
    panel.setFill( this.NONE );
    panel.setAnchor( this.WEST );
    panel.add( slopeCheckBox );
    if ( includeStandardLines ) {
      panel.add( positiveCheckBox );
      panel.add( negativeCheckBox );
    }
    panel.add( new JPanel()
    {

      {
        this.add( hideLinesCheckBox );
      }
    }
  )
    ;
    this.addChild( new ControlPanelNode( panel, this.GRAPH_CONTROL_PANEL ) );
    linesVisible.addObserver( new VoidFunction1()
    {

      apply: function( visible ) {
        slopeCheckBox.setEnabled( visible );
        positiveCheckBox.setEnabled( visible );
        negativeCheckBox.setEnabled( visible );
      }
    }
  )
    ;
    yEqualsXVisible.addObserver( new VoidFunction1()
    {

      apply: function( visible ) {
        if ( visible ) {
          standardLines.add( this.Y_EQUALS_X_LINE );
        }
        else {
          standardLines.remove( this.Y_EQUALS_X_LINE );
        }
      }
    }
  )
    ;
    yEqualsNegativeXVisible.addObserver( new VoidFunction1()
    {

      apply: function( visible ) {
        if ( visible ) {
          standardLines.add( this.Y_EQUALS_NEGATIVE_X_LINE );
        }
        else {
          standardLines.remove( this.Y_EQUALS_NEGATIVE_X_LINE );
        }
      }
    }
  )
    ;
    standardLines.addElementAddedObserver( new VoidFunction1()
    {

      apply: function( line ) {
        if ( line == this.Y_EQUALS_X_LINE ) {
          yEqualsXVisible.set( true );
        }
        else if ( line == this.Y_EQUALS_NEGATIVE_X_LINE ) {
          yEqualsNegativeXVisible.set( true );
        }
      }
    }
  )
    ;
    standardLines.addElementRemovedObserver( new VoidFunction1()
    {

      apply: function( line ) {
        if ( line == this.Y_EQUALS_X_LINE ) {
          yEqualsXVisible.set( false );
        }
        else if ( line == this.Y_EQUALS_NEGATIVE_X_LINE ) {
          yEqualsNegativeXVisible.set( false );
        }
      }
    }
  )
    ;
  }

  return inherit( PNode, GraphControls, {

    main: function( args ) {
      var linesVisible = new Property( true );
      var slopeVisible = new Property( true );
      var standardLines = new ObservableList();
      var panel1 = new GraphControls( linesVisible, slopeVisible, standardLines );
      var panel2 = new GraphControls( linesVisible, slopeVisible );
      var canvas = new PhetPCanvas();
      canvas.setPreferredSize( new Dimension( 300, 600 ) );
      canvas.getLayer().addChild( panel1 );
      canvas.getLayer().addChild( panel2 );
      panel1.setOffset( 50, 50 );
      panel2.setOffset( panel1.getXOffset(), panel1.getFullBoundsReference().getMaxY() + 20 );
      var frame = new JFrame();
      frame.setContentPane( canvas );
      frame.pack();
      frame.setDefaultCloseOperation( this.EXIT_ON_CLOSE );
      frame.setVisible( true );
    },


  } );

} );

