// Copyright 2002-2013, University of Colorado Boulder

define( function( require ) {
  'use strict';

  var inherit = require( 'PHET_CORE/inherit' );

  var Color = require( 'Color' );
  var Font = require( 'Font' );
  var Vector2D = require( 'Vector2D' );
  var ModelViewTransform = require( 'ModelViewTransform' );
  var PhetFont = require( 'PhetFont' );
  var DoubleArrowNode = require( 'DoubleArrowNode' );
  var ZeroOffsetNode = require( 'ZeroOffsetNode' );
  var Graph = require( 'Graph' );
  var Line = require( 'Line' );
  var PNode = require( 'PNode' );
  var PDimension = require( 'PDimension' );
  var PComposite = require( 'PComposite' );

//Class declaration for LineNode

  /**
   * Constructor
   *
   * @param line  the line to draw
   * @param graph the graph to draw it on
   * @param mvt   the transform between model and view coordinate frames
   */
  /**
   * Constructor for LineNode
   * @param {Line} line
   * @param {Graph} graph
   * @param {ModelViewTransform} mvt
   */
  function LineNode( line, graph, mvt ) {
    this.line = line;
    var xExtent = mvt.viewToModelDeltaX( LINE_EXTENT );
    var yExtent = Math.abs( mvt.viewToModelDeltaY( LINE_EXTENT ) );
    var tailX, tailY, tipX, tipY;
    if ( this.run == 0 ) {
      tailX = this.x1;
      tailY = this.yRange.getMax() + yExtent;
      tipX = this.x1;
      tipY = this.yRange.getMin() - yExtent;
    }
    else if ( this.rise == 0 ) {
      tailX = this.xRange.getMin() - xExtent;
      tailY = this.y1;
      tipX = this.xRange.getMax() + yExtent;
      tipY = this.y1;
    }
    else {
      tailX = this.xRange.getMin() - xExtent;
      tailY = line.solveY( tailX );
      if ( tailY < this.yRange.getMin() - yExtent ) {
        tailX = line.solveX( this.yRange.getMin() - yExtent );
        tailY = line.solveY( tailX );
      }
      else if ( tailY > this.yRange.getMax() + yExtent ) {
        tailX = line.solveX( this.yRange.getMax() + yExtent );
        tailY = line.solveY( tailX );
      }
      tipX = this.xRange.getMax() + xExtent;
      tipY = line.solveY( tipX );
      if ( tipY < this.yRange.getMin() - yExtent ) {
        tipX = line.solveX( this.yRange.getMin() - yExtent );
        tipY = line.solveY( tipX );
      }
      else if ( tipY > this.yRange.getMax() + yExtent ) {
        tipX = line.solveX( this.yRange.getMax() + yExtent );
        tipY = line.solveY( tipX );
      }
    }
    tailLocation = new Vector2D( mvt.modelToViewX( tailX ), mvt.modelToViewY( tailY ) );
    tipLocation = new Vector2D( mvt.modelToViewX( tipX ), mvt.modelToViewY( tipY ) );
    arrowNode = new DoubleArrowNode( tailLocation.toPoint2D(), tipLocation.toPoint2D(), ARROW_HEAD_SIZE.getHeight(), ARROW_HEAD_SIZE.getWidth(), LINE_THICKNESS );
    arrowNode.setPaint( this.color );
    arrowNode.setStroke( null );
    this.addChild( arrowNode );
    equationParentNode = new PNode();
    this.addChild( equationParentNode );
    equationParentNode.setOffset( tipLocation.toPoint2D() );
    equationParentNode.setRotation( line.undefinedSlope() ? this.PI / 2 : -Math.atan( line.getSlope() ) );
    this.updateEquation( line, EQUATION_FONT, this.color );
  }

  return inherit( PComposite, LineNode, {

    createEquationNode: function( line, font, color ) {
      return new PNode();
    },


    setEquationVisible: function( visible ) {
      equationParentNode.setVisible( visible );
    },


    updateColor: function( color ) {
      arrowNode.setPaint( color );
      this.updateEquation( line, EQUATION_FONT, color );
    },


    updateEquation: function( line, font, color ) {
      equationParentNode.removeAllChildren();
      var zeroOffsetNode = new ZeroOffsetNode( this.createEquationNode( line, font, color ) );
      equationParentNode.addChild( new ZeroOffsetNode( zeroOffsetNode ) );
      if ( line.undefinedSlope() ) {
        if ( this.rise < 0 ) {
          equationParentNode.setOffset( tipLocation.toPoint2D() );
          zeroOffsetNode.setOffset( -zeroOffsetNode.getFullBoundsReference().getWidth() - 30, -zeroOffsetNode.getFullBoundsReference().getHeight() - 12 );
        }
        else {
          equationParentNode.setOffset( tailLocation.toPoint2D() );
          zeroOffsetNode.setOffset( 30, -zeroOffsetNode.getFullBoundsReference().getHeight() - 12 );
        }
      }
      else if ( this.rise <= 0 ) {
        if ( this.run >= 0 ) {
          equationParentNode.setOffset( tipLocation.toPoint2D() );
          zeroOffsetNode.setOffset( -zeroOffsetNode.getFullBoundsReference().getWidth() - 30, -zeroOffsetNode.getFullBoundsReference().getHeight() - 12 );
        }
        else {
          equationParentNode.setOffset( tailLocation.toPoint2D() );
          zeroOffsetNode.setOffset( 30, -zeroOffsetNode.getFullBoundsReference().getHeight() - 12 );
        }
      }
      else {
        if ( this.run > 0 ) {
          equationParentNode.setOffset( tipLocation.toPoint2D() );
          zeroOffsetNode.setOffset( -zeroOffsetNode.getFullBoundsReference().getWidth() - 30, 10 );
        }
        else {
          equationParentNode.setOffset( tailLocation.toPoint2D() );
          zeroOffsetNode.setOffset( 30, 10 );
        }
      }
    },


  } );

} );

