// Copyright 2002-2013, University of Colorado Boulder

define( function( require ) {
  'use strict';

  var inherit = require( 'PHET_CORE/inherit' );

  var Color = require( 'Color' );
  var Font = require( 'Font' );
  var Point2D = require( 'Point2D' );
  var Rectangle2D = require( 'Rectangle2D' );
  var MessageFormat = require( 'MessageFormat' );
  var NumberFormat = require( 'NumberFormat' );
  var MathUtil = require( 'MathUtil' );
  var Vector2D = require( 'Vector2D' );
  var Property = require( 'Property' );
  var ParameterSet = require( 'ParameterSet' );
  var UserComponentTypes = require( 'UserComponentTypes' );
  var DefaultDecimalFormat = require( 'DefaultDecimalFormat' );
  var RichSimpleObserver = require( 'RichSimpleObserver' );
  var ModelViewTransform = require( 'ModelViewTransform' );
  var PhetFont = require( 'PhetFont' );
  var PhetPNode = require( 'PhetPNode' );
  var CursorHandler = require( 'CursorHandler' );
  var SimSharingDragHandler = require( 'SimSharingDragHandler' );
  var LGColors = require( 'LGColors' );
  var Images = require( 'Images' );
  var Strings = require( 'Strings' );
  var ParameterKeys = require( 'ParameterKeys' );
  var UserComponents = require( 'UserComponents' );
  var Graph = require( 'Graph' );
  var Line = require( 'Line' );
  var PointTool = require( 'PointTool' );
  var Orientation = require( 'Orientation' );
  var PNode = require( 'PNode' );
  var PInputEvent = require( 'PInputEvent' );
  var PImage = require( 'PImage' );
  var PPath = require( 'PPath' );
  var PText = require( 'PText' );

//Class declaration for PointToolNode

  /**
   * Constructor
   *
   * @param pointTool    the point tool
   * @param mvt          transform between model and view coordinate frames
   * @param graph        the graph that the point tool "sees"
   * @param dragBounds   drag bounds, in view coordinate frame
   * @param linesVisible whether lines are currently visible
   */
  /**
   * Constructor for PointToolNode
   * @param {PointTool} pointTool
   * @param {ModelViewTransform} mvt
   * @param {Graph} graph
   * @param {Rectangle2D} dragBounds
   * @param {Property<Boolean>} linesVisible
   */
  function PointToolNode( pointTool, mvt, graph, dragBounds, linesVisible ) {
    this( this.location.get(), this.orientation, this.POINT_TOOL_BACKGROUND_NORMAL_COLOR );
    var observer = new RichSimpleObserver()
    {

      update: function() {
        var location = this.location.get();
        this.setOffset( mvt.modelToView( location ).toPoint2D() );
        if ( graph.contains( location ) ) {
          this.setCoordinates( location );
          if ( linesVisible.get() ) {
            var onLine = this.onLine.get();
            this.setForeground( onLine == null ? this.POINT_TOOL_FOREGROUND_NORMAL_COLOR : this.POINT_TOOL_FOREGROUND_HIGHLIGHT_COLOR );
            this.setBackground( onLine == null ? this.POINT_TOOL_BACKGROUND_NORMAL_COLOR : this.color );
          }
          else {
            this.setForeground( this.POINT_TOOL_FOREGROUND_NORMAL_COLOR );
            this.setBackground( this.POINT_TOOL_BACKGROUND_NORMAL_COLOR );
          }
        }
        else {
          this.setCoordinates( this.POINT_UNKNOWN );
          this.setForeground( this.POINT_TOOL_FOREGROUND_NORMAL_COLOR );
          this.setBackground( this.POINT_TOOL_BACKGROUND_NORMAL_COLOR );
        }
      }
    }
    ;
    observer.observe( this.location, this.onLine, linesVisible );
    this.addInputEventListener( new CursorHandler() );
    this.addInputEventListener( new PointToolDragHandler( this, this.location, mvt, graph, dragBounds ) );
  }

  /**
   * This constructor creates a node that is independent of the model.
   * This was needed so that we could easily generate images of the point tool, for inclusion in the game reward.
   */
  /**
   * Constructor for PointToolNode
   * @param {Vector2D} point
   * @param {Orientation} orientation
   * @param {Color} background
   */
  function PointToolNode( point, orientation, background ) {
    bodyNode = new PImage( this.POINT_TOOL_BODY );
    var tipNode = new PImage( this.POINT_TOOL_TIP );
    tipNode.setPickable( false );
    var backgroundMargin = 5;
    backgroundNode = new PPath( new Rectangle2D.Double( 0, 0, bodyNode.getFullBoundsReference().getWidth() - (2 * backgroundMargin), bodyNode.getFullBoundsReference().getHeight() - (2 * backgroundMargin) ) );
    backgroundNode.setStroke( null );
    backgroundNode.setOffset( bodyNode.getOffset() );
    backgroundNode.setPickable( false );
    valueNode = new PText( "?" );
    valueNode.setFont( new PhetFont( this.BOLD, 15 ) );
    valueNode.setPickable( false );
    this.addChild( tipNode );
    this.addChild( backgroundNode );
    this.addChild( bodyNode );
    this.addChild( valueNode );
    if ( orientation == this.DOWN ) {
      tipNode.setOffset( -tipNode.getFullBoundsReference().getWidth() / 2, -tipNode.getFullBoundsReference().getHeight() );
      bodyNode.setOffset( -bodyNode.getFullBoundsReference().getWidth() / 2, tipNode.getFullBoundsReference().getMinY() - bodyNode.getFullBoundsReference().getHeight() );
      backgroundNode.setOffset( bodyNode.getXOffset() + backgroundMargin, bodyNode.getYOffset() + backgroundMargin );
      valueNode.setOffset( 0, bodyNode.getFullBoundsReference().getMinY() + COORDINATES_Y_CENTER - (valueNode.getFullBoundsReference().getHeight() / 2) );
    }
    else {
      tipNode.rotate( this.PI );
      tipNode.setOffset( tipNode.getFullBoundsReference().getWidth() / 2, tipNode.getFullBoundsReference().getHeight() );
      bodyNode.setOffset( -bodyNode.getFullBoundsReference().getWidth() / 2, tipNode.getFullBoundsReference().getMaxY() );
      backgroundNode.setOffset( bodyNode.getFullBoundsReference().getMinX() + backgroundMargin, bodyNode.getFullBoundsReference().getMinY() + backgroundMargin );
      valueNode.setOffset( 0, bodyNode.getFullBoundsReference().getMaxY() - COORDINATES_Y_CENTER - (valueNode.getFullBoundsReference().getHeight() / 2) );
    }
    this.setCoordinates( point );
    this.setBackground( background );
  }

  return inherit( PhetPNode, PointToolNode, {

    setCoordinates: function( point ) {
      this.setCoordinates( MessageFormat.format( this.POINT_XY, COORDINATES_FORMAT.format( point.getX() ), COORDINATES_FORMAT.format( point.getY() ) ) );
    },


    setCoordinates: function( s ) {
      valueNode.setText( s );
      valueNode.setOffset( bodyNode.getFullBoundsReference().getCenterX() - (valueNode.getFullBoundsReference().getWidth() / 2), valueNode.getYOffset() );
    },


    setForeground: function( color ) {
      valueNode.setTextPaint( color );
    },


    setBackground: function( color ) {
      backgroundNode.setPaint( color );
    },


  } );

} );

