// Copyright 2002-2013, University of Colorado Boulder

define( function( require ) {
  'use strict';

  var inherit = require( 'PHET_CORE/inherit' );

  var BasicStroke = require( 'BasicStroke' );
  var Color = require( 'Color' );
  var Font = require( 'Font' );
  var Line2D = require( 'Line2D' );
  var Point2D = require( 'Point2D' );
  var NumberFormat = require( 'NumberFormat' );
  var Icon = require( 'Icon' );
  var ImageIcon = require( 'ImageIcon' );
  var Property = require( 'Property' );
  var DefaultDecimalFormat = require( 'DefaultDecimalFormat' );
  var VoidFunction1 = require( 'VoidFunction1' );
  var ModelViewTransform = require( 'ModelViewTransform' );
  var DoubleGeneralPath = require( 'DoubleGeneralPath' );
  var PhetFont = require( 'PhetFont' );
  var PadBoundsNode = require( 'PadBoundsNode' );
  var PhetPPath = require( 'PhetPPath' );
  var LGColors = require( 'LGColors' );
  var Line = require( 'Line' );
  var LineFormsModel = require( 'LineFormsModel' );
  var SlopeInterceptModel = require( 'SlopeInterceptModel' );
  var PNode = require( 'PNode' );
  var PPath = require( 'PPath' );
  var PDimension = require( 'PDimension' );
  var PComposite = require( 'PComposite' );

//Class declaration for SlopeToolNode

  /**
   * Constructor for SlopeToolNode
   * @param {Property<Line>} line
   * @param {ModelViewTransform} mvt
   */
  function SlopeToolNode( line, mvt ) {
    this.setPickable( false );
    this.setChildrenPickable( false );
    line.addObserver( new VoidFunction1()
    {

      apply: function( line ) {
        this.update( line, mvt );
      }
    }
  )
    ;
  }

  return inherit( PComposite, SlopeToolNode, {

    update: function( line, mvt ) {
      this.removeAllChildren();
      if ( this.rise == 0 || this.run == 0 ) {
        return;
      }
      var gridXSpacing = mvt.modelToViewDeltaX( 1 );
      var gridYSpacing = mvt.modelToViewDeltaY( 1 );
      var p1View = mvt.modelToView( this.x1, this.y1 );
      var p2View = mvt.modelToView( this.x2, this.y2 );
      var offsetFactor = 0.6;
      var delimiterLengthFactor = 0.5;
      var riseLineNode, riseTailDelimiterNode, riseTipDelimiterNode, riseValueNode;
      {
        riseValueNode = new NumberBackgroundNode( this.rise, VALUE_FORMAT, VALUE_FONT, VALUE_TEXT_COLOR, this.SLOPE, VALUE_X_MARGIN, VALUE_Y_MARGIN, VALUE_CORNER_RADIUS );
        var xOffset = offsetFactor * gridXSpacing;
        var riseDelimiterLength = delimiterLengthFactor * gridXSpacing;
        var tipFudgeY = (this.rise > 0) ? STROKE_WIDTH : -STROKE_WIDTH;
        var arrowX;
        if ( this.run > 0 ) {
          arrowX = p1View.getX() - xOffset;
          riseLineNode = new ArrowNode( arrowX, p1View.getY(), arrowX, p2View.getY() + tipFudgeY );
          riseValueNode.setOffset( riseLineNode.getFullBoundsReference().getMinX() - riseValueNode.getFullBoundsReference().getWidth() - VALUE_X_SPACING, riseLineNode.getFullBoundsReference().getCenterY() - (riseValueNode.getFullBoundsReference().getHeight() / 2) );
        }
        else {
          arrowX = p1View.getX() + xOffset;
          riseLineNode = new ArrowNode( arrowX, p1View.getY(), arrowX, p2View.getY() + tipFudgeY );
          riseValueNode.setOffset( riseLineNode.getFullBoundsReference().getMaxX() + VALUE_X_SPACING, riseLineNode.getFullBoundsReference().getCenterY() - (riseValueNode.getFullBoundsReference().getHeight() / 2) );
        }
        riseTailDelimiterNode = new DimensionalDelimiterNode( arrowX - (riseDelimiterLength / 2), p1View.getY(), arrowX + (riseDelimiterLength / 2), p1View.getY() );
        riseTipDelimiterNode = new DimensionalDelimiterNode( arrowX - (riseDelimiterLength / 2), p2View.getY(), arrowX + (riseDelimiterLength / 2), p2View.getY() );
      }
      var runLineNode, runTailDelimiterNode, runTipDelimiterNode, runValueNode;
      {
        runValueNode = new NumberBackgroundNode( this.run, VALUE_FORMAT, VALUE_FONT, VALUE_TEXT_COLOR, this.SLOPE, VALUE_X_MARGIN, VALUE_Y_MARGIN, VALUE_CORNER_RADIUS );
        var yOffset = offsetFactor * gridYSpacing;
        var runDelimiterLength = delimiterLengthFactor * gridYSpacing;
        var tipFudgeX = (this.run > 0) ? -1 : 1;
        var arrowY;
        if ( this.rise > 0 ) {
          arrowY = p2View.getY() + yOffset;
          runLineNode = new ArrowNode( p1View.getX(), arrowY, p2View.getX() + tipFudgeX, arrowY );
          runValueNode.setOffset( runLineNode.getFullBoundsReference().getCenterX() - (runValueNode.getFullBoundsReference().getWidth() / 2), runLineNode.getFullBoundsReference().getMinY() - riseValueNode.getFullBoundsReference().getHeight() - VALUE_Y_SPACING );
        }
        else {
          arrowY = p2View.getY() - yOffset;
          runLineNode = new ArrowNode( p1View.getX(), arrowY, p2View.getX() + tipFudgeX, arrowY );
          runValueNode.setOffset( runLineNode.getFullBoundsReference().getCenterX() - (runValueNode.getFullBoundsReference().getWidth() / 2), runLineNode.getFullBoundsReference().getMaxY() + VALUE_Y_SPACING );
        }
        runTailDelimiterNode = new DimensionalDelimiterNode( p1View.getX(), arrowY - (runDelimiterLength / 2), p1View.getX(), arrowY + (runDelimiterLength / 2) );
        runTipDelimiterNode = new DimensionalDelimiterNode( p2View.getX(), arrowY - (runDelimiterLength / 2), p2View.getX(), arrowY + (runDelimiterLength / 2) );
      }
      this.addChild( riseTailDelimiterNode );
      this.addChild( riseTipDelimiterNode );
      this.addChild( riseLineNode );
      this.addChild( runTailDelimiterNode );
      this.addChild( runTipDelimiterNode );
      this.addChild( runLineNode );
      this.addChild( riseValueNode );
      this.addChild( runValueNode );
    },


    createIcon: function( width ) {
      var parentNode = new PadBoundsNode();
      var model = new SlopeInterceptModel();
      this.interactiveLine.set( Line.createSlopeIntercept( 1, 2, 0 ) );
      var slopeToolNode = new SlopeToolNode( this.interactiveLine, this.mvt );
      parentNode.addChild( slopeToolNode );
      var lineNode = new PPath( new Line2D.Double( slopeToolNode.getFullBoundsReference().getMinX() + (0.4 * slopeToolNode.getFullBoundsReference().getWidth()), slopeToolNode.getFullBoundsReference().getMaxY(), slopeToolNode.getFullBoundsReference().getMaxX(), slopeToolNode.getFullBoundsReference().getMinY() + (0.5 * slopeToolNode.getFullBoundsReference().getHeight()) ) );
      lineNode.setStroke( new BasicStroke( 1
      f, this.CAP_BUTT, this.JOIN_BEVEL, 0, new float[]
      { 6, 6 }
      ,
      0
      ))
      ;
      parentNode.addChild( lineNode );
      parentNode.scale( width / parentNode.getFullBoundsReference().getWidth() );
      return new ImageIcon( parentNode.toImage() );
    },


  } );

} );

