// Copyright 2002-2013, University of Colorado Boulder

define( function( require ) {
  'use strict';

  var inherit = require( 'PHET_CORE/inherit' );

  var Point2D = require( 'Point2D' );
  var ArrayList = require( 'ArrayList' );
  var MathUtil = require( 'MathUtil' );
  var Vector2D = require( 'Vector2D' );
  var Property = require( 'Property' );
  var IUserComponent = require( 'IUserComponent' );
  var IUserComponentType = require( 'IUserComponentType' );
  var DoubleRange = require( 'DoubleRange' );
  var ModelViewTransform = require( 'ModelViewTransform' );
  var SimSharingDragHandler = require( 'SimSharingDragHandler' );
  var PInputEvent = require( 'PInputEvent' );

//Class declaration for PointDragHandler

  /**
   * Constructor
   *
   * @param userComponent   sim-sharing component identifier
   * @param componentType   sim-sharing component type
   * @param manipulatorNode the node being manipulated by the user
   * @param mvt             transform between model and view coordinate frames
   * @param point           the point being manipulated
   * @param otherPoints     other points that the manipulated point can't match
   * @param x1Range         range of the x coordinate
   * @param y1Range         range of the y coordinate
   */
  /**
   * Constructor for PointDragHandler
   * @param {IUserComponent} userComponent
   * @param {IUserComponentType} componentType
   * @param {LineManipulatorNode} manipulatorNode
   * @param {ModelViewTransform} mvt
   * @param {Property<Vector2D>} point
   * @param {ArrayList<Property<Vector2D>>} otherPoints
   * @param {Property<DoubleRange>} x1Range
   * @param {Property<DoubleRange>} y1Range
   */
  function PointDragHandler( userComponent, componentType, manipulatorNode, mvt, point, otherPoints, x1Range, y1Range ) {
    super( userComponent, componentType, true );
    this.manipulatorNode = manipulatorNode;
    this.mvt = mvt;
    this.point = point;
    this.otherPoints = otherPoints;
    this.x1Range = x1Range;
    this.y1Range = y1Range;
  }

  return inherit( SimSharingDragHandler, PointDragHandler, {

    startDrag: function( event ) {
      super.startDrag( event );
      var pMouse = event.getPositionRelativeTo( manipulatorNode.getParent() );
      clickXOffset = pMouse.getX() - mvt.modelToViewDeltaX( this.x );
      clickYOffset = pMouse.getY() - mvt.modelToViewDeltaY( this.y );
    },


    drag: function( event ) {
      super.drag( event );
      var pMouse = event.getPositionRelativeTo( manipulatorNode.getParent() );
      var x = MathUtil.roundHalfUp( MathUtil.clamp( mvt.viewToModelDeltaX( pMouse.getX() - clickXOffset ), x1Range.get() ) );
      var y = MathUtil.roundHalfUp( MathUtil.clamp( mvt.viewToModelDeltaY( pMouse.getY() - clickYOffset ), y1Range.get() ) );
      var p = new Vector2D( x, y );
      var same = false;
      for ( var pOther :
      otherPoints
      )
      {
        if ( p.equals( pOther.get() ) ) {
          same = true;
          break;
        }
      }
      if ( !same ) {
        point.set( new Vector2D( x, y ) );
      }
    },


  } );

} );

