// Copyright 2002-2013, University of Colorado Boulder

define( function( require ) {
  'use strict';

  var inherit = require( 'PHET_CORE/inherit' );

  var Point2D = require( 'Point2D' );
  var MathUtil = require( 'MathUtil' );
  var Property = require( 'Property' );
  var IUserComponent = require( 'IUserComponent' );
  var IUserComponentType = require( 'IUserComponentType' );
  var DoubleRange = require( 'DoubleRange' );
  var ModelViewTransform = require( 'ModelViewTransform' );
  var Line = require( 'Line' );
  var PInputEvent = require( 'PInputEvent' );

//Class declaration for X1Y1DragHandler

  /**
   * Constructor
   *
   * @param userComponent   sim-sharing component identifier
   * @param componentType   sim-sharing component type
   * @param manipulatorNode the node being manipulated by the user
   * @param mvt             transform between model and view coordinate frames
   * @param line            the line being manipulated
   * @param x1Range         range of the x coordinate
   * @param y1Range         range of the y coordinate
   * @param constantSlope   true: slope is constant, false: (x2,y2) is constant
   */
  /**
   * Constructor for X1Y1DragHandler
   * @param {IUserComponent} userComponent
   * @param {IUserComponentType} componentType
   * @param {LineManipulatorNode} manipulatorNode
   * @param {ModelViewTransform} mvt
   * @param {Property<Line>} line
   * @param {Property<DoubleRange>} x1Range
   * @param {Property<DoubleRange>} y1Range
   * @param {boolean} constantSlope
   */
  function X1Y1DragHandler( userComponent, componentType, manipulatorNode, mvt, line, x1Range, y1Range, constantSlope ) {
    super( userComponent, componentType, manipulatorNode, mvt, line );
    this.x1Range = x1Range;
    this.y1Range = y1Range;
    this.constantSlope = constantSlope;
  }

  return inherit( LineManipulatorDragHandler, X1Y1DragHandler, {

    startDrag: function( event ) {
      super.startDrag( event );
      var pMouse = event.getPositionRelativeTo( manipulatorNode.getParent() );
      clickXOffset = pMouse.getX() - mvt.modelToViewDeltaX( this.x1 );
      clickYOffset = pMouse.getY() - mvt.modelToViewDeltaY( this.y1 );
    },


    drag: function( event ) {
      super.drag( event );
      var pMouse = event.getPositionRelativeTo( manipulatorNode.getParent() );
      var x1 = MathUtil.roundHalfUp( MathUtil.clamp( mvt.viewToModelDeltaX( pMouse.getX() - clickXOffset ), x1Range.get() ) );
      var y1 = MathUtil.roundHalfUp( MathUtil.clamp( mvt.viewToModelDeltaY( pMouse.getY() - clickYOffset ), y1Range.get() ) );
      if ( constantSlope ) {
        line.set( Line.createPointSlope( x1, y1, this.rise, this.run, this.color ) );
      }
      else if ( !(x1 == this.x2 && y1 == this.y2) ) {
        line.set( new Line( x1, y1, this.x2, this.y2, this.color ) );
      }
    },


  } );

} );

