// Copyright 2002-2013, University of Colorado Boulder

define( function( require ) {
  'use strict';

  var inherit = require( 'PHET_CORE/inherit' );

  var Cursor = require( 'Cursor' );
  var Image = require( 'Image' );
  var ActionEvent = require( 'ActionEvent' );
  var ActionListener = require( 'ActionListener' );
  var Timer = require( 'Timer' );
  var ObservableProperty = require( 'ObservableProperty' );
  var Property = require( 'Property' );
  var SimSharingManager = require( 'SimSharingManager' );
  var IUserComponent = require( 'IUserComponent' );
  var ParameterKey = require( 'ParameterKey' );
  var ParameterKeys = require( 'ParameterKeys' );
  var ParameterSet = require( 'ParameterSet' );
  var UserActions = require( 'UserActions' );
  var UserComponentTypes = require( 'UserComponentTypes' );
  var RichSimpleObserver = require( 'RichSimpleObserver' );
  var SimpleObserver = require( 'SimpleObserver' );
  var Function0 = require( 'Function0' );
  var VoidFunction0 = require( 'VoidFunction0' );
  var VoidFunction1 = require( 'VoidFunction1' );
  var DynamicCursorHandler = require( 'DynamicCursorHandler' );
  var PNode = require( 'PNode' );
  var PBasicInputEventHandler = require( 'PBasicInputEventHandler' );
  var PInputEvent = require( 'PInputEvent' );
  var PImage = require( 'PImage' );

//Class declaration for SpinnerButtonNode

  /**
   * Constructor
   *
   * @param userComponent    component identifier for user data-collection message
   * @param images           set of images for the button states
   * @param pressed          is the button pressed?
   * @param inside           is the mouse inside the bounds of the button?
   * @param enabled          property that controls whether the button is enabled
   * @param newValueFunction function that computes the new value when the button fires
   */
  /**
   * Constructor for SpinnerButtonNode
   * @param {IUserComponent} userComponent
   * @param {SpinnerStateIndicator<Image>} images
   * @param {Property<Boolean>} pressed
   * @param {Property<Boolean>} inside
   * @param {ObservableProperty<Boolean>} enabled
   * @param {Property<T>} value
   * @param {Function0<T>} newValueFunction
   */
  function SpinnerButtonNode( userComponent, images, pressed, inside, enabled, value, newValueFunction ) {
    var imageNode = new PImage();
    this.addChild( imageNode );
    var cursorHandler = new DynamicCursorHandler();
    this.addInputEventListener( cursorHandler );
    enabled.addObserver( new VoidFunction1()
    {

      apply: function( enabled ) {
        cursorHandler.setCursor( enabled ? this.HAND_CURSOR : this.DEFAULT_CURSOR );
      }
    }
  )
    ;
    new RichSimpleObserver()
    {

      update: function() {
        imageNode.setImage( new Function0()
        {

          apply: function() {
            if ( !enabled.get() ) {
              return this.disabled;
            }
            else if ( pressed.get() ) {
              return this.pressed;
            }
            else if ( inside.get() ) {
              return this.highlighted;
            }
            else {
              return this.inactive;
            }
          }
        }
      .
        apply()
      )
        ;
      }
    }
  .
    observe( enabled, pressed, inside );
    {
      var buttonAction = new VoidFunction0()
      {

        apply: function() {
          var newValue = newValueFunction.apply();
          SimSharingManager.sendUserMessage( userComponent, this.button, this.pressed, ParameterSet.parameterSet( this.value, newValue.toString() ).with( new ParameterKey( "spinContinuously" ), spinContinuously ) );
          value.set( newValue );
        }
      }
      ;
      var timer = new Timer( 200, new ActionListener()
      {

        actionPerformed: function( e ) {
          if ( enabled.get() ) {
            spinContinuously = true;
            buttonAction.apply();
          }
        }
      }
    )
      {

        {
          this.setInitialDelay( 500 );
        }
      }
      ;
      pressed.addObserver( new SimpleObserver()
      {

        update: function() {
          if ( pressed.get() ) {
            if ( enabled.get() ) {
              buttonAction.apply();
              timer.start();
            }
          }
          else {
            timer.stop();
            spinContinuously = false;
          }
        }
      }
    )
      ;
    }
    imageNode.addInputEventListener( new PBasicInputEventHandler()
    {

      mousePressed: function( event ) {
        pressed.set( true );
      }

      mouseReleased: function( event ) {
        pressed.set( false );
      }

      mouseEntered: function( event ) {
        inside.set( true );
      }

      mouseExited: function( event ) {
        inside.set( false );
      }
    }
  )
    ;
  }

  return inherit( PNode, SpinnerButtonNode, {

  } );

} );

