// Copyright 2002-2013, University of Colorado Boulder

define( function( require ) {
  'use strict';

  var inherit = require( 'PHET_CORE/inherit' );

  var Color = require( 'Color' );
  var Cursor = require( 'Cursor' );
  var Dimension = require( 'Dimension' );
  var Font = require( 'Font' );
  var GradientPaint = require( 'GradientPaint' );
  var Image = require( 'Image' );
  var Paint = require( 'Paint' );
  var Shape = require( 'Shape' );
  var Rectangle2D = require( 'Rectangle2D' );
  var RoundRectangle2D = require( 'RoundRectangle2D' );
  var DecimalFormat = require( 'DecimalFormat' );
  var NumberFormat = require( 'NumberFormat' );
  var JFrame = require( 'JFrame' );
  var WindowConstants = require( 'WindowConstants' );
  var BooleanProperty = require( 'BooleanProperty' );
  var CompositeProperty = require( 'CompositeProperty' );
  var ObservableProperty = require( 'ObservableProperty' );
  var Property = require( 'Property' );
  var IUserComponent = require( 'IUserComponent' );
  var UserComponentChain = require( 'UserComponentChain' );
  var DoubleRange = require( 'DoubleRange' );
  var RichSimpleObserver = require( 'RichSimpleObserver' );
  var SimpleObserver = require( 'SimpleObserver' );
  var Function0 = require( 'Function0' );
  var VoidFunction1 = require( 'VoidFunction1' );
  var ColorUtils = require( 'ColorUtils' );
  var PhetFont = require( 'PhetFont' );
  var ShapeUtils = require( 'ShapeUtils' );
  var PhetPCanvas = require( 'PhetPCanvas' );
  var DynamicCursorHandler = require( 'DynamicCursorHandler' );
  var PhetPText = require( 'PhetPText' );
  var UserComponents = require( 'UserComponents' );
  var BackgroundColors = require( 'BackgroundColors' );
  var DownButtonImages = require( 'DownButtonImages' );
  var InterceptColors = require( 'InterceptColors' );
  var UpButtonImages = require( 'UpButtonImages' );
  var PNode = require( 'PNode' );
  var PBasicInputEventHandler = require( 'PBasicInputEventHandler' );
  var PInputEvent = require( 'PInputEvent' );
  var PPath = require( 'PPath' );
  var PText = require( 'PText' );
  var PDimension = require( 'PDimension' );

//Class declaration for SpinnerNode

  /**
   * Constructor for SpinnerNode
   * @param {IUserComponent} userComponent
   * @param {Property<Double>} value
   * @param {Property<DoubleRange>} range
   * @param {SpinnerStateIndicator<Color>} colors
   * @param {PhetFont} font
   * @param {NumberFormat} format
   */
  function SpinnerNode( userComponent, value, range, colors, font, format ) {
    this( userComponent, value, range, colors, font, format, new Function0()
    {

      apply: function() {
        return value.get() + 1;
      }
    }
  ,
    new Function0()
    {

      apply: function() {
        return value.get() - 1;
      }
    }
  )
    ;
  }

  /**
   * Constructor for SpinnerNode
   * @param {IUserComponent} userComponent
   * @param {Property<Double>} value
   * @param {Property<DoubleRange>} range
   * @param {SpinnerStateIndicator<Color>} colors
   * @param {PhetFont} font
   * @param {NumberFormat} format
   * @param {Function0<Double>} upFunction
   * @param {Function0<Double>} downFunction
   */
  function SpinnerNode( userComponent, value, range, colors, font, format, upFunction, downFunction ) {
    this( userComponent, value, range, new UpButtonImages( colors ), new DownButtonImages( colors ), new BackgroundColors( this.highlighted, this.pressed ), font, format, upFunction, downFunction );
  }

  /**
   * Fully-specified constructor.
   *
   * @param userComponent    user component in data-collection messages
   * @param value            the value property that is being manipulated by the spinner
   * @param range            range of the value
   * @param upButtonImages   images for the "up" button states
   * @param downButtonImages images for the "down" button states
   * @param backgroundColors colors for the background state
   * @param font             font used to display the value
   * @param format           displayed format of the value
   * @param upFunction       function called when the "up" (increment) button is pressed
   * @param downFunction     function called when the "down" (decrement) button is pressed
   */
  /**
   * Constructor for SpinnerNode
   * @param {IUserComponent} userComponent
   * @param {Property<Double>} value
   * @param {Property<DoubleRange>} range
   * @param {SpinnerStateIndicator<Image>} upButtonImages
   * @param {SpinnerStateIndicator<Image>} downButtonImages
   * @param {SpinnerStateIndicator<Color>} backgroundColors
   * @param {PhetFont} font
   * @param {NumberFormat} format
   * @param {Function0<Double>} upFunction
   * @param {Function0<Double>} downFunction
   */
  function SpinnerNode( userComponent, value, range, upButtonImages, downButtonImages, backgroundColors, font, format, upFunction, downFunction ) {
    var upPressed = new BooleanProperty( false );
    var downPressed = new BooleanProperty( false );
    var upEnabled = new CompositeProperty( new Function0()
    {

      apply: function() {
        return value.get() < range.get().getMax();
      }
    }
  ,
    value, range
  )
    ;
    var upInside = new BooleanProperty( false );
    var downInside = new BooleanProperty( false );
    var downEnabled = new CompositeProperty( new Function0()
    {

      apply: function() {
        return value.get() > range.get().getMin();
      }
    }
  ,
    value, range
  )
    ;
    var textNode = new PhetPText( font );
    textNode.setPickable( false );
    textNode.setText( "-20" );
    var xMargin = 3;
    var yMargin = 3;
    var backgroundWidth = textNode.getFullBoundsReference().getWidth() + (2 * xMargin);
    var backgroundHeight = textNode.getFullBoundsReference().getHeight() + (2 * yMargin);
    var backgroundOverlap = 0.5;
    var backgroundCornerRadius = 10;
    var backgroundShape = new RoundRectangle2D.Double( 0, 0, backgroundWidth, backgroundHeight, backgroundCornerRadius, backgroundCornerRadius );
    var upBackgroundNode = new PPath()
    {

      {
        this.setPathTo( ShapeUtils.subtract( backgroundShape, new Rectangle2D.Double( 0, (backgroundHeight / 2) - backgroundOverlap, backgroundWidth, backgroundHeight ) ) );
        this.setStroke( null );
        this.setPaint( this.inactive );
        this.addInputEventListener( new BackgroundMouseHandler( this, new PDimension( backgroundWidth, backgroundHeight ), upPressed, upInside, upEnabled, backgroundColors ) );
      }
    }
    ;
    var downBackgroundNode = new PPath()
    {

      {
        this.setPathTo( ShapeUtils.subtract( backgroundShape, new Rectangle2D.Double( 0, 0, backgroundWidth, (backgroundHeight / 2) - backgroundOverlap ) ) );
        this.setStroke( null );
        this.setPaint( this.inactive );
        this.addInputEventListener( new BackgroundMouseHandler( this, new PDimension( backgroundWidth, backgroundHeight ), downPressed, downInside, downEnabled, backgroundColors ) );
      }
    }
    ;
    var upButton = new SpinnerButtonNode( UserComponentChain.chain( userComponent, "up" ), upButtonImages, upPressed, upInside, upEnabled, value, upFunction );
    var downButton = new SpinnerButtonNode( UserComponentChain.chain( userComponent, "down" ), downButtonImages, downPressed, downInside, downEnabled, value, downFunction );
    this.addChild( upBackgroundNode );
    this.addChild( downBackgroundNode );
    this.addChild( upButton );
    this.addChild( downButton );
    this.addChild( textNode );
    upBackgroundNode.setOffset( 0, 0 );
    downBackgroundNode.setOffset( 0, 0 );
    textNode.setOffset( 0, (backgroundHeight - textNode.getFullBoundsReference().getHeight()) / 2 );
    upButton.setOffset( upBackgroundNode.getFullBoundsReference().getCenterX() - (upButton.getFullBoundsReference().getWidth() / 2), upBackgroundNode.getFullBoundsReference().getMinY() - upButton.getFullBoundsReference().getHeight() - 1 );
    downButton.setOffset( downBackgroundNode.getFullBoundsReference().getCenterX() - (downButton.getFullBoundsReference().getWidth() / 2), downBackgroundNode.getFullBoundsReference().getMaxY() );
    value.addObserver( new SimpleObserver()
    {

      update: function() {
        textNode.setText( format.format( value.get() ) );
        textNode.setOffset( (backgroundWidth - textNode.getFullBoundsReference().getWidth()) / 2, textNode.getYOffset() );
      }
    }
  )
    ;
  }

  return inherit( PNode, SpinnerNode, {

    main: function( args ) {
      var range = new Property( new DoubleRange( -10, 10, 0 ) );
      var value = new Property( range.get().getDefault() );
      var node = new SpinnerNode( this.interceptSpinner, value, range, new InterceptColors(), new PhetFont( this.BOLD, 24 ), new DecimalFormat( "0" ) );
      node.setOffset( 200, 200 );
      var canvas = new PhetPCanvas();
      canvas.setPreferredSize( new Dimension( 600, 400 ) );
      canvas.getLayer().addChild( node );
      var frame = new JFrame();
      frame.setContentPane( canvas );
      frame.pack();
      frame.setDefaultCloseOperation( this.EXIT_ON_CLOSE );
      frame.setVisible( true );
    },


  } );

} );

