// Copyright 2002-2013, University of Colorado Boulder

define( function( require ) {
  'use strict';

  var inherit = require( 'PHET_CORE/inherit' );

  var Point2D = require( 'Point2D' );
  var GameAudioPlayer = require( 'GameAudioPlayer' );
  var Vector2D = require( 'Vector2D' );
  var Resettable = require( 'Resettable' );
  var Property = require( 'Property' );
  var IntegerRange = require( 'IntegerRange' );
  var SimpleObserver = require( 'SimpleObserver' );
  var ModelViewTransform = require( 'ModelViewTransform' );
  var Strings = require( 'Strings' );
  var Graph = require( 'Graph' );
  var Line = require( 'Line' );
  var PointTool = require( 'PointTool' );
  var Orientation = require( 'Orientation' );
  var LineGameConstants = require( 'LineGameConstants' );
  var ChallengeNode = require( 'ChallengeNode' );
  var PDimension = require( 'PDimension' );

//Class declaration for Challenge

  /**
   * Constructor.
   * @param title title that is visible to the user
   * @param description brief description of the challenge, visible in dev versions
   * @param answer the correct answer
   * @param equationForm see EquationForm
   * @param manipulationMode see ManipulationMode
   * @param xRange range of the graph's x axis
   * @param yRange range of the graph's y axis
   * @param originOffset offset of the origin (center of the graph) in view coordinates
   * @param pointToolLocation1 location of point tool in model coordinates
   * @param pointToolLocation2 location of point tool in model coordinates
   */
  /**
   * Constructor for Challenge
   * @param {String} title
   * @param {String} description
   * @param {Line} answer
   * @param {EquationForm} equationForm
   * @param {ManipulationMode} manipulationMode
   * @param {IntegerRange} xRange
   * @param {IntegerRange} yRange
   * @param {Point2D} originOffset
   * @param {Vector2D} pointToolLocation1
   * @param {Vector2D} pointToolLocation2
   */
  function Challenge( title, description, answer, equationForm, manipulationMode, xRange, yRange, originOffset, pointToolLocation1, pointToolLocation2 ) {
    assert( !answer.undefinedSlope() );
    this.title = title;
    this.description = description;
    this.answer = answer.withColor( this.ANSWER_COLOR );
    this.guess = new Property( this.createInitialGuess( answer, manipulationMode ) );
    this.equationForm = equationForm;
    this.manipulationMode = manipulationMode;
    var mvtScale = GRAPH_WIDTH / xRange.getLength();
    mvt = ModelViewTransform.createOffsetScaleMapping( originOffset, mvtScale, -mvtScale );
    graph = new Graph( xRange, yRange );
    this.pointTool1 = new PointTool( pointToolLocation1, this.UP, this.lines );
    this.pointTool2 = new PointTool( pointToolLocation2, this.DOWN, this.lines );
    this.guess.addObserver( new SimpleObserver()
    {

      update: function() {
        this.updateGraphLines();
      }
    }
  )
    ;
  }

  return inherit( Object, Challenge, {

    reset: function() {
      guess.reset();
      pointTool1.reset();
      pointTool2.reset();
      this.setAnswerVisible( false );
    },


    /**
     * Creates the view component for the challenge.
     * @param model the game model
     * @param challengeSize dimensions of the view rectangle that is available for rendering the challenge
     * @param audioPlayer the audio player, for providing audio feedback during game play
     */
    createView: function( model, challengeSize, audioPlayer );
  ,


  isCorrect: function() {
    return answer.same( guess.get() );
  }
  ,


  setAnswerVisible: function( visible ) {
    answerVisible = visible;
    this.updateGraphLines();
  }
  ,


  updateGraphLines: function()
  ;
  ,


  createInitialGuess: function( answer, manipulationMode ) {
    if ( manipulationMode == this.SLOPE ) {
      return Line.createPointSlope( this.x1, this.y1, 1, 1, this.GUESS_COLOR );
    }
    else if ( manipulationMode == this.INTERCEPT ) {
      return Line.createSlopeIntercept( this.rise, this.run, 0, this.GUESS_COLOR );
    }
    else if ( manipulationMode == this.POINT ) {
      return Line.createPointSlope( 0, 0, this.rise, this.run, this.GUESS_COLOR );
    }
    else if ( manipulationMode == this.THREE_POINTS ) {
      return null;
    }
    else {
      return this.Y_EQUALS_X_LINE.withColor( this.GUESS_COLOR );
    }
  }
  ,


  createTitle: function( defaultTitle, manipulationMode ) {
    if ( manipulationMode == this.SLOPE ) {
      return this.SET_THE_SLOPE;
    }
    else if ( manipulationMode == this.INTERCEPT ) {
      return this.SET_THE_Y_INTERCEPT;
    }
    else if ( manipulationMode == this.POINT ) {
      return this.SET_THE_POINT;
    }
    else if ( manipulationMode == this.THREE_POINTS ) {
      return this.PUT_POINTS_ON_LINE;
    }
    else {
      return defaultTitle;
    }
  }
  ,


} );

} )
;

