// Copyright 2002-2013, University of Colorado Boulder

define( function( require ) {
  'use strict';

  var inherit = require( 'PHET_CORE/inherit' );

  var Point2D = require( 'Point2D' );
  var ArrayList = require( 'ArrayList' );
  var Collections = require( 'Collections' );
  var IntegerRange = require( 'IntegerRange' );
  var Fraction = require( 'Fraction' );
  var Line = require( 'Line' );

//Class declaration for ChallengeFactory2
  return inherit( ChallengeFactory, ChallengeFactory2, {

    /**
     * Creates challenges for this game level.
     * @param xRange range of the graph's x axis
     * @param yRange range of the graph's y axis
     * @return list of challenges
     */
    createChallenges: function( xRange, yRange ) {
      var challenges = new ArrayList();
      var slopeLists = this.createSlopeLists();
      var slopeListIndices = this.rangeToList( new IntegerRange( 0, slopeLists.size() - 1 ) );
      var yInterceptRange = new IntegerRange( -10, 10 );
      var yInterceptLists = new ArrayList()
      {

        {
          assert( yInterceptRange.getMin() < 0 && yInterceptRange.getMax() > 0 );
          this.add( this.rangeToList( new IntegerRange( yInterceptRange.getMin(), -1 ) ) );
          this.add( this.rangeToList( new IntegerRange( 1, yInterceptRange.getMax() ) ) );
        }
      ,

      }
      ;
      var yInterceptListIndices = this.rangeToList( new IntegerRange( 0, yInterceptLists.size() - 1 ) );
      var pointSlopeManipulationModes = new ArrayList()
      {

        {
          this.add( this.POINT );
          this.add( this.SLOPE );
        }
      ,

      }
      ;
      {
        var slope = fractionChooser.chooseFromLists( slopeLists, slopeListIndices );
        var yIntercept = integerChooser.chooseFromLists( yInterceptLists );
        challenges.add( new GraphTheLine( "1 of 3 required slopes", Line.createSlopeIntercept( this.numerator, this.denominator, yIntercept ), this.SLOPE_INTERCEPT, this.SLOPE, xRange, yRange ) );
      }
      {
        var slope = fractionChooser.chooseFromLists( slopeLists );
        var yIntercept = integerChooser.chooseFromLists( yInterceptLists, yInterceptListIndices );
        challenges.add( new GraphTheLine( "1 of 2 required y-intercepts", Line.createSlopeIntercept( this.numerator, this.denominator, yIntercept ), this.SLOPE_INTERCEPT, this.INTERCEPT, xRange, yRange ) );
      }
      {
        var slope = fractionChooser.chooseFromLists( slopeLists, slopeListIndices );
        var yIntercept = integerChooser.chooseFromLists( yInterceptLists );
        challenges.add( new MakeTheEquation( "2 of 3 requires slopes", Line.createSlopeIntercept( this.numerator, this.denominator, yIntercept ), this.SLOPE_INTERCEPT, this.SLOPE, xRange, yRange ) );
      }
      {
        var slope = fractionChooser.chooseFromLists( slopeLists );
        var yIntercept = integerChooser.chooseFromLists( yInterceptLists, yInterceptListIndices );
        challenges.add( new MakeTheEquation( "2 of 2 required y-intercepts", Line.createSlopeIntercept( this.numerator, this.denominator, yIntercept ), this.SLOPE_INTERCEPT, this.INTERCEPT, xRange, yRange ) );
      }
      {
        var manipulationMode = manipulationModeChooser.choose( pointSlopeManipulationModes );
        var slope;
        var description;
        if ( manipulationMode == this.SLOPE ) {
          slope = fractionChooser.chooseFromLists( slopeLists, slopeListIndices );
          description = "random choice of slope manipulation, 3 of 3 required slopes";
        }
        else {
          slope = fractionChooser.chooseFromLists( slopeLists );
          description = "random choice of point manipulation";
        }
        var point = this.choosePointForSlope( slope, xRange, yRange );
        challenges.add( new GraphTheLine( description, Line.createPointSlope( point.getX(), point.getY(), this.numerator, this.denominator ), this.POINT_SLOPE, manipulationMode, xRange, yRange ) );
      }
      {
        var manipulationMode = manipulationModeChooser.choose( pointSlopeManipulationModes );
        var slope;
        var description;
        if ( manipulationMode == this.SLOPE ) {
          slope = fractionChooser.chooseFromLists( slopeLists, slopeListIndices );
          description = "slope manipulation because Graph-the-Line uses point, 3 of 3 required slopes";
        }
        else {
          slope = fractionChooser.chooseFromLists( slopeLists );
          description = "point manipulation because Graph-the-Line uses slope";
        }
        var point = this.choosePointForSlope( slope, xRange, yRange );
        challenges.add( new MakeTheEquation( description, Line.createPointSlope( point.getX(), point.getY(), this.numerator, this.denominator ), this.POINT_SLOPE, manipulationMode, xRange, yRange ) );
      }
      Collections.shuffle( challenges );
      return challenges;
    },


    createPositiveFractionalSlopes: function() {
      return new ArrayList()
      {

        {
          this.add( new Fraction( 1, 4 ) );
          this.add( new Fraction( 1, 5 ) );
          this.add( new Fraction( 1, 6 ) );
          this.add( new Fraction( 1, 7 ) );
          this.add( new Fraction( 2, 5 ) );
          this.add( new Fraction( 3, 5 ) );
          this.add( new Fraction( 2, 7 ) );
          this.add( new Fraction( 3, 7 ) );
          this.add( new Fraction( 4, 7 ) );
          this.add( new Fraction( 5, 2 ) );
          this.add( new Fraction( 3, 2 ) );
          this.add( new Fraction( 7, 2 ) );
          this.add( new Fraction( 7, 3 ) );
          this.add( new Fraction( 7, 4 ) );
        }
      ,

      }
      ;
    },


    createSlopeLists: function() {
      return new ArrayList()
      {

        {
          this.add( new ArrayList()
          {

            {
              this.add( new Fraction( 1, 1 ) );
              this.add( new Fraction( 2, 1 ) );
              this.add( new Fraction( 3, 1 ) );
              this.add( new Fraction( 4, 1 ) );
              this.add( new Fraction( 5, 1 ) );
              this.add( new Fraction( -1, 1 ) );
              this.add( new Fraction( -2, 1 ) );
              this.add( new Fraction( -3, 1 ) );
              this.add( new Fraction( -4, 1 ) );
              this.add( new Fraction( -5, 1 ) );
            }
          ,

          }
        )
          ;
          this.add( this.createPositiveFractionalSlopes() );
          this.add( new ArrayList()
          {

            {
              this.add( new Fraction( -1, 2 ) );
              this.add( new Fraction( -1, 3 ) );
              this.add( new Fraction( -1, 4 ) );
              this.add( new Fraction( -1, 5 ) );
              this.add( new Fraction( -2, 3 ) );
              this.add( new Fraction( -3, 4 ) );
              this.add( new Fraction( -2, 5 ) );
              this.add( new Fraction( -3, 5 ) );
              this.add( new Fraction( -4, 5 ) );
              this.add( new Fraction( -3, 2 ) );
              this.add( new Fraction( -4, 3 ) );
              this.add( new Fraction( -5, 2 ) );
              this.add( new Fraction( -5, 3 ) );
              this.add( new Fraction( -5, 4 ) );
            }
          ,

          }
        )
          ;
        }
      ,

      }
      ;
    },


  } );

} );

