// Copyright 2002-2013, University of Colorado Boulder

define( function( require ) {
  'use strict';

  var inherit = require( 'PHET_CORE/inherit' );

  var Color = require( 'Color' );
  var Point2D = require( 'Point2D' );
  var ArrayList = require( 'ArrayList' );
  var Collections = require( 'Collections' );
  var IntegerRange = require( 'IntegerRange' );
  var Fraction = require( 'Fraction' );
  var Line = require( 'Line' );

//Class declaration for ChallengeFactory5
  return inherit( ChallengeFactory, ChallengeFactory5, {

    /**
     * Creates challenges for this game level.
     * @param xRange range of the graph's x axis
     * @param yRange range of the graph's y axis
     * @return list of challenges
     */
    createChallenges: function( xRange, yRange ) {
      var challenges = new ArrayList();
      var yIntercepts = this.rangeToList( new IntegerRange( -10, 10 ) );
      {
        var yIntercept = integerChooser.choose( yIntercepts );
        challenges.add( new MakeTheEquation( "slope=0", Line.createSlopeIntercept( 0, 1, yIntercept ), this.SLOPE_INTERCEPT, this.SLOPE_INTERCEPT, xRange, yRange ) );
      }
      {
        var yIntercept = integerChooser.choose( yIntercepts );
        challenges.add( new GraphTheLine( "slope=0", Line.createSlopeIntercept( 0, 1, yIntercept ), this.SLOPE_INTERCEPT, this.SLOPE_INTERCEPT, xRange, yRange ) );
      }
      {
        var equationForms = new ArrayList()
        {

          {
            this.add( this.SLOPE_INTERCEPT );
            this.add( this.POINT_SLOPE );
          }
        ,

        }
        ;
        var equationForm = equationFormChooser.choose( equationForms );
        var range = new IntegerRange( -5, 5 );
        var xList = this.rangeToList( range );
        var yList = this.rangeToList( range );
        var x1 = (equationForm == this.SLOPE_INTERCEPT) ? 0 : integerChooser.choose( xList );
        var y1 = integerChooser.choose( yList );
        var x2 = integerChooser.choose( xList );
        if ( x2 == x1 ) {
          x2 = integerChooser.choose( xList );
        }
        var y2 = integerChooser.choose( yList );
        var slope = (y2 - y1) / (x2 - x1);
        if ( slope == 1 || slope == -1 ) {
          y2 = integerChooser.choose( yList );
        }
        var line = new Line( x1, y1, x2, y2, this.BLACK );
        if ( equationForm == this.SLOPE_INTERCEPT ) {
          challenges.add( new GraphTheLine( "random choice of slope-intercept, points in [-5,5]", line, this.SLOPE_INTERCEPT, this.SLOPE_INTERCEPT, xRange, yRange ) );
        }
        else {
          challenges.add( new GraphTheLine( "random choice of point-slope, points in [-5,5]", line, this.POINT_SLOPE, this.POINT_SLOPE, xRange, yRange ) );
        }
      }
      {
        var equationForms = new ArrayList()
        {

          {
            this.add( this.SLOPE_INTERCEPT );
            this.add( this.POINT_SLOPE );
          }
        ,

        }
        ;
        var equationForm = equationFormChooser.choose( equationForms );
        var excludedSlopes = new ArrayList()
        {

          {
            this.add( new Fraction( 1, 1 ) );
            this.add( new Fraction( 2, 1 ) );
            this.add( new Fraction( 1, 2 ) );
            this.add( new Fraction( 1, 3 ) );
            this.add( new Fraction( 1, 4 ) );
            this.add( new Fraction( 2, 3 ) );
          }
        ,

        }
        ;
        var rise, run;
        {
          var riseList = this.rangeToList( yRange );
          var runList = this.rangeToList( xRange );
          rise = integerChooser.choose( riseList );
          run = integerChooser.choose( runList );
          var excluded = true;
          while ( excluded && runList.size() > 0 ) {
            excluded = false;
            for ( var slope :
            excludedSlopes
          )
            {
              if ( run == 0 || slope.toDecimal() == new Fraction( rise, run ).toDecimal() ) {
                excluded = true;
                run = integerChooser.choose( runList );
                break;
              }
            }
          }
          if ( excluded ) {
            run = 5;
          }
        }
        assert( run != 0 );
        var p = this.choosePointForSlope( new Fraction( rise, run ), xRange, yRange );
        var x1 = (equationForm == this.SLOPE_INTERCEPT) ? 0 : p.getX();
        var y1 = p.getY();
        var x2 = x1 + run;
        var y2 = y1 + rise;
        var line = new Line( x1, y1, x2, y2, this.BLACK );
        if ( equationForm == this.SLOPE_INTERCEPT ) {
          challenges.add( new GraphTheLine( "random choice of slope-intercept, some excluded slopes", line, this.SLOPE_INTERCEPT, this.SLOPE_INTERCEPT, xRange, yRange ) );
        }
        else {
          challenges.add( new GraphTheLine( "random choice of point-slope, some excluded slopes", line, this.POINT_SLOPE, this.POINT_SLOPE, xRange, yRange ) );
        }
      }
      this.addPlaceThePointsChallenges( challenges, integerChooser, xRange, yRange );
      Collections.shuffle( challenges );
      return challenges;
    },


    /**
     * Adds 2 "Place the Point" challenges, 1 slope-intercept form, 1 point-slope form.
     * Pulled out into a method that can be reused in level=6.
     *
     * @param challenges add challenges to this list
     * @param integerChooser for choosing random integers
     * @param xRange range of the graph's x axis
     * @param yRange range of the graph's y axis
     */
    addPlaceThePointsChallenges: function( challenges, integerChooser, xRange, yRange ) {
      var range = new IntegerRange( -5, 5 );
      var xList = this.rangeToList( range );
      var yList = this.rangeToList( range );
      var riseList = this.rangeToList( range );
      var runList = this.rangeToList( range );
      riseList.remove( new Integer( 0 ) );
      runList.remove( new Integer( 0 ) );
      {
        var x1 = 0;
        var y1 = integerChooser.choose( yList );
        var rise = integerChooser.choose( riseList );
        var run = integerChooser.choose( runList );
        if ( Math.abs( rise / run ) == 1 ) {
          run = integerChooser.choose( runList );
        }
        challenges.add( new PlaceThePoints( "slope-intercept, random points", new Line( x1, y1, x1 + run, y1 + rise, this.BLACK ), this.SLOPE_INTERCEPT, xRange, yRange ) );
      }
      {
        var x1 = integerChooser.choose( xList );
        var y1 = integerChooser.choose( yList );
        var rise = integerChooser.choose( riseList );
        var run = integerChooser.choose( runList );
        if ( Math.abs( rise / run ) == 1 ) {
          run = integerChooser.choose( runList );
        }
        challenges.add( new PlaceThePoints( "point-slope, random points", new Line( x1, y1, x1 + run, y1 + rise, this.BLACK ), this.POINT_SLOPE, xRange, yRange ) );
      }
    },


  } );

} );

