// Copyright 2002-2013, University of Colorado Boulder

define( function( require ) {
  'use strict';

  var inherit = require( 'PHET_CORE/inherit' );

  var BasicStroke = require( 'BasicStroke' );
  var Color = require( 'Color' );
  var Font = require( 'Font' );
  var ActionEvent = require( 'ActionEvent' );
  var ActionListener = require( 'ActionListener' );
  var Rectangle2D = require( 'Rectangle2D' );
  var MessageFormat = require( 'MessageFormat' );
  var GameAudioPlayer = require( 'GameAudioPlayer' );
  var PhetApplication = require( 'PhetApplication' );
  var BooleanProperty = require( 'BooleanProperty' );
  var VoidFunction1 = require( 'VoidFunction1' );
  var PhetFont = require( 'PhetFont' );
  var PhetPNode = require( 'PhetPNode' );
  var FaceNode = require( 'FaceNode' );
  var PhetPText = require( 'PhetPText' );
  var TextButtonNode = require( 'TextButtonNode' );
  var Strings = require( 'Strings' );
  var Line = require( 'Line' );
  var EquationNode = require( 'EquationNode' );
  var PointToolNode = require( 'PointToolNode' );
  var LineGameConstants = require( 'LineGameConstants' );
  var Challenge = require( 'Challenge' );
  var EquationForm = require( 'EquationForm' );
  var LineGameModel = require( 'LineGameModel' );
  var PlayState = require( 'PlayState' );
  var PointSlopeEquationNode = require( 'PointSlopeEquationNode' );
  var SlopeInterceptEquationNode = require( 'SlopeInterceptEquationNode' );
  var PNode = require( 'PNode' );
  var PText = require( 'PText' );
  var PDimension = require( 'PDimension' );

//Class declaration for ChallengeNode

  /**
   * Constructor
   * @param challenge the challenge
   * @param model the game model
   * @param challengeSize dimensions of the view rectangle that is available for rendering the challenge
   * @param audioPlayer the audio player, for providing audio feedback during game play
   */
  /**
   * Constructor for ChallengeNode
   * @param {Challenge} challenge
   * @param {LineGameModel} model
   * @param {PDimension} challengeSize
   * @param {GameAudioPlayer} audioPlayer
   */
  function ChallengeNode( challenge, model, challengeSize, audioPlayer ) {
    var descriptionNode = new PhetPText( this.description, new PhetFont( 12 ), this.BLACK );
    descriptionNode.setOffset( 5, 5 );
    faceNode = new FaceNode( this.FACE_DIAMETER, this.FACE_COLOR, new BasicStroke( 1
    f
  ),
    this.FACE_COLOR.darker(), this.BLACK, this.BLACK
  )
    ;
    pointsAwardedNode = new PhetPText( "", this.POINTS_AWARDED_FONT, this.POINTS_AWARDED_COLOR );
    var buttonFont = this.BUTTON_FONT;
    var buttonBackground = this.BUTTON_COLOR;
    checkButton = new TextButtonNode( this.CHECK, buttonFont, buttonBackground );
    var tryAgainButton = new TextButtonNode( this.TRY_AGAIN, buttonFont, buttonBackground );
    var showAnswerButton = new TextButtonNode( this.SHOW_ANSWER, buttonFont, buttonBackground );
    var nextButton = new TextButtonNode( this.NEXT, buttonFont, buttonBackground );
    var skipButton = new TextButtonNode( "dev: Skip", new PhetFont( this.BOLD, 12 ), this.WHITE );
    var replayButton = new TextButtonNode( "dev: Replay", new PhetFont( this.BOLD, 12 ), this.WHITE );
    var pointToolDragBounds = new Rectangle2D.Double( 0, 0, challengeSize.getWidth(), challengeSize.getHeight() );
    var pointToolNode1 = new PointToolNode( this.pointTool1, this.mvt, this.graph, pointToolDragBounds, new BooleanProperty( true ) );
    var pointToolNode2 = new PointToolNode( this.pointTool2, this.mvt, this.graph, pointToolDragBounds, new BooleanProperty( true ) );
    pointToolNode1.scale( this.POINT_TOOL_SCALE );
    pointToolNode2.scale( this.POINT_TOOL_SCALE );
    var pointToolParent = new PNode();
    pointToolParent.addChild( pointToolNode1 );
    pointToolParent.addChild( pointToolNode2 );
    subclassParent = new PNode();
    {
      this.addChild( subclassParent );
      if ( PhetApplication.getInstance().isDeveloperControlsEnabled() ) {
        this.addChild( descriptionNode );
      }
      this.addChild( checkButton );
      this.addChild( tryAgainButton );
      this.addChild( showAnswerButton );
      this.addChild( nextButton );
      if ( PhetApplication.getInstance().isDeveloperControlsEnabled() ) {
        this.addChild( skipButton );
        this.addChild( replayButton );
      }
      this.addChild( pointToolParent );
      this.addChild( faceNode );
      this.addChild( pointsAwardedNode );
    }
    {
      var buttonCenterX = (challengeSize.getWidth() / 2);
      var buttonCenterY = challengeSize.getHeight() - checkButton.getFullBoundsReference().getHeight() - 30;
      checkButton.setOffset( buttonCenterX - (checkButton.getFullBoundsReference().getWidth() / 2), buttonCenterY );
      tryAgainButton.setOffset( buttonCenterX - (tryAgainButton.getFullBoundsReference().getWidth() / 2), buttonCenterY );
      showAnswerButton.setOffset( buttonCenterX - (showAnswerButton.getFullBoundsReference().getWidth() / 2), buttonCenterY );
      nextButton.setOffset( buttonCenterX - (nextButton.getFullBoundsReference().getWidth() / 2), buttonCenterY );
      skipButton.setOffset( nextButton.getFullBoundsReference().getCenterX() - (skipButton.getFullBoundsReference().getWidth() / 2), nextButton.getFullBoundsReference().getMaxY() + 2 );
      replayButton.setOffset( nextButton.getFullBoundsReference().getCenterX() - (replayButton.getFullBoundsReference().getWidth() / 2), nextButton.getFullBoundsReference().getMaxY() + 2 );
    }
    checkButton.addActionListener( new ActionListener()
    {

      actionPerformed: function( e ) {
        if ( challenge.isCorrect() ) {
          faceNode.smile();
          audioPlayer.correctAnswer();
          var points = model.computePoints( this.state.get() == this.FIRST_CHECK ? 1 : 2 );
          this.score.set( this.score.get() + points );
          pointsAwardedNode.setText( MessageFormat.format( this.POINTS_AWARDED, String.valueOf( points ) ) );
          pointsAwardedNode.setOffset( faceNode.getFullBoundsReference().getMaxX() + 10, faceNode.getFullBoundsReference().getCenterY() - (pointsAwardedNode.getFullBoundsReference().getHeight() / 2) );
          this.state.set( this.NEXT );
        }
        else {
          faceNode.frown();
          audioPlayer.wrongAnswer();
          pointsAwardedNode.setText( "" );
          if ( this.state.get() == this.FIRST_CHECK ) {
            this.state.set( this.TRY_AGAIN );
          }
          else {
            this.state.set( this.SHOW_ANSWER );
          }
        }
      }
    }
  )
    ;
    tryAgainButton.addActionListener( new ActionListener()
    {

      actionPerformed: function( e ) {
        this.state.set( this.SECOND_CHECK );
      }
    }
  )
    ;
    showAnswerButton.addActionListener( new ActionListener()
    {

      actionPerformed: function( e ) {
        this.state.set( this.NEXT );
      }
    }
  )
    ;
    nextButton.addActionListener( new ActionListener()
    {

      actionPerformed: function( e ) {
        this.state.set( this.FIRST_CHECK );
      }
    }
  )
    ;
    skipButton.addActionListener( new ActionListener()
    {

      actionPerformed: function( e ) {
        model.skipCurrentChallenge();
      }
    }
  )
    ;
    replayButton.addActionListener( new ActionListener()
    {

      actionPerformed: function( e ) {
        model.replayCurrentChallenge();
      }
    }
  )
    ;
    this.state.addObserver( new VoidFunction1()
    {

      apply: function( state ) {
        faceNode.setVisible( state == this.TRY_AGAIN || state == this.SHOW_ANSWER || (state == this.NEXT && challenge.isCorrect()) );
        pointsAwardedNode.setVisible( faceNode.getVisible() && challenge.isCorrect() );
        checkButton.setVisible( state == this.FIRST_CHECK || state == this.SECOND_CHECK );
        tryAgainButton.setVisible( state == this.TRY_AGAIN );
        showAnswerButton.setVisible( state == this.SHOW_ANSWER );
        nextButton.setVisible( state == this.NEXT );
        skipButton.setVisible( !nextButton.getVisible() );
        replayButton.setVisible( nextButton.getVisible() );
      }
    }
  )
    ;
  }

  return inherit( PhetPNode, ChallengeNode, {

    createEquationNode: function( equationForm, line, font, color ) {
      if ( equationForm == this.SLOPE_INTERCEPT ) {
        return new SlopeInterceptEquationNode( line, font, color );
      }
      else if ( equationForm == this.POINT_SLOPE ) {
        return new PointSlopeEquationNode( line, font, color );
      }
      else {
        throw new IllegalArgumentException( "unsupported equation form: " + equationForm );
      }
    },


  } );

} );

