// Copyright 2002-2013, University of Colorado Boulder

define( function( require ) {
  'use strict';

  var inherit = require( 'PHET_CORE/inherit' );

  var Color = require( 'Color' );
  var Font = require( 'Font' );
  var GameAudioPlayer = require( 'GameAudioPlayer' );
  var VoidFunction0 = require( 'VoidFunction0' );
  var VoidFunction1 = require( 'VoidFunction1' );
  var PhetFont = require( 'PhetFont' );
  var PhetPText = require( 'PhetPText' );
  var Strings = require( 'Strings' );
  var Line = require( 'Line' );
  var LineGameConstants = require( 'LineGameConstants' );
  var Challenge = require( 'Challenge' );
  var GraphTheLine = require( 'GraphTheLine' );
  var LineGameModel = require( 'LineGameModel' );
  var ManipulationMode = require( 'ManipulationMode' );
  var PlayState = require( 'PlayState' );
  var PNode = require( 'PNode' );
  var PDimension = require( 'PDimension' );

//Class declaration for GraphTheLineNode

  /**
   * Constructor
   * @param challenge the challenge
   * @param model the game model
   * @param challengeSize dimensions of the view rectangle that is available for rendering the challenge
   * @param audioPlayer the audio player, for providing audio feedback during game play
   */
  /**
   * Constructor for GraphTheLineNode
   * @param {GraphTheLine} challenge
   * @param {LineGameModel} model
   * @param {PDimension} challengeSize
   * @param {GameAudioPlayer} audioPlayer
   */
  function GraphTheLineNode( challenge, model, challengeSize, audioPlayer ) {
    super( challenge, model, challengeSize, audioPlayer );
    var titleNode = new PhetPText( this.title, this.TITLE_FONT, this.TITLE_COLOR );
    var maxTitleWidth = 0.45 * challengeSize.getWidth();
    if ( titleNode.getFullBoundsReference().getWidth() > maxTitleWidth ) {
      titleNode.scale( maxTitleWidth / titleNode.getFullBoundsReference().getWidth() );
    }
    var boxSize = new PDimension( 0.4 * challengeSize.getWidth(), 0.3 * challengeSize.getHeight() );
    var answerBoxNode = new EquationBoxNode( this.LINE_TO_GRAPH, this.color, boxSize, this.createEquationNode( this.equationForm, this.answer, this.STATIC_EQUATION_FONT, this.color ) );
    guessBoxNode = new EquationBoxNode( this.YOUR_LINE, this.BLACK, boxSize, new PNode() );
    graphNode = this.createGraphNode( challenge );
    graphNode.setGuessPointVisible( this.manipulationMode == this.SLOPE );
    subclassParent.addChild( titleNode );
    subclassParent.addChild( graphNode );
    subclassParent.addChild( answerBoxNode );
    subclassParent.addChild( guessBoxNode );
    {
      answerBoxNode.setOffset( (challengeSize.getWidth() / 2) - answerBoxNode.getFullBoundsReference().getWidth() - 50, this.mvt.modelToViewY( 0 ) - answerBoxNode.getFullBoundsReference().getHeight() - 10 );
      faceNode.setOffset( answerBoxNode.getFullBoundsReference().getCenterX() - (faceNode.getFullBoundsReference().getWidth() / 2), checkButton.getFullBoundsReference().getMaxY() - faceNode.getFullBoundsReference().getHeight() );
      titleNode.setOffset( answerBoxNode.getFullBoundsReference().getMinX(), answerBoxNode.getFullBoundsReference().getMinY() - titleNode.getFullBoundsReference().getHeight() - 20 );
    }
    var updateIcons = new VoidFunction0()
    {

      apply: function() {
        answerBoxNode.setCorrectIconVisible( this.state.get() == this.NEXT );
        guessBoxNode.setCorrectIconVisible( this.state.get() == this.NEXT && challenge.isCorrect() );
        guessBoxNode.setIncorrectIconVisible( this.state.get() == this.NEXT && !challenge.isCorrect() );
      }
    }
    ;
    this.guess.addObserver( new VoidFunction1()
    {

      apply: function( line ) {
        subclassParent.removeChild( guessBoxNode );
        var equationNode = (line == null) ? NOT_A_LINE : this.createEquationNode( this.equationForm, line, this.STATIC_EQUATION_FONT, this.color );
        var color = (line == null) ? this.GUESS_COLOR : this.color;
        guessBoxNode = new EquationBoxNode( this.YOUR_LINE, color, boxSize, equationNode );
        guessBoxNode.setOffset( answerBoxNode.getXOffset(), this.mvt.modelToViewY( 0 ) + 10 );
        subclassParent.addChild( guessBoxNode );
        guessBoxNode.setVisible( this.state.get() == this.NEXT );
        updateIcons.apply();
      }
    }
  )
    ;
    this.state.addObserver( new VoidFunction1()
    {

      apply: function( state ) {
        graphNode.setPickable( state == this.FIRST_CHECK || state == this.SECOND_CHECK || (state == this.NEXT && !challenge.isCorrect()) );
        graphNode.setChildrenPickable( graphNode.getPickable() );
        graphNode.setAnswerVisible( state == this.NEXT );
        if ( state == this.NEXT && !challenge.isCorrect() ) {
          guessBoxNode.setVisible( true );
          graphNode.setAnswerPointVisible( true );
          graphNode.setGuessPointVisible( true );
          graphNode.setSlopeToolVisible( true );
        }
        updateIcons.apply();
      }
    }
  )
    ;
  }

  return inherit( ChallengeNode, GraphTheLineNode, {

    createGraphNode: function( challenge ) {
      if ( this.manipulationMode == this.POINT || this.manipulationMode == this.SLOPE || this.manipulationMode == this.POINT_SLOPE ) {
        return new PointSlopeGraphNode( challenge );
      }
      else if ( this.manipulationMode == this.INTERCEPT || this.manipulationMode == this.SLOPE_INTERCEPT ) {
        assert( this.answer.getYIntercept().isInteger() );
        return new SlopeInterceptGraphNode( challenge );
      }
      else if ( this.manipulationMode == this.TWO_POINTS ) {
        return new TwoPointsGraphNode( challenge );
      }
      else {
        throw new IllegalArgumentException( "unsupported manipulationMode: " + this.manipulationMode );
      }
    },


  } );

} );

