// Copyright 2002-2013, University of Colorado Boulder

define( function( require ) {
  'use strict';

  var inherit = require( 'PHET_CORE/inherit' );

  var Color = require( 'Color' );
  var GameAudioPlayer = require( 'GameAudioPlayer' );
  var PhetApplication = require( 'PhetApplication' );
  var Property = require( 'Property' );
  var DoubleRange = require( 'DoubleRange' );
  var SimpleObserver = require( 'SimpleObserver' );
  var VoidFunction0 = require( 'VoidFunction0' );
  var VoidFunction1 = require( 'VoidFunction1' );
  var PhetFont = require( 'PhetFont' );
  var PhetPText = require( 'PhetPText' );
  var Strings = require( 'Strings' );
  var Graph = require( 'Graph' );
  var Line = require( 'Line' );
  var EquationNode = require( 'EquationNode' );
  var LineGameConstants = require( 'LineGameConstants' );
  var EquationForm = require( 'EquationForm' );
  var LineGameModel = require( 'LineGameModel' );
  var MakeTheEquation = require( 'MakeTheEquation' );
  var ManipulationMode = require( 'ManipulationMode' );
  var PlayState = require( 'PlayState' );
  var PointSlopeEquationNode = require( 'PointSlopeEquationNode' );
  var SlopeInterceptEquationNode = require( 'SlopeInterceptEquationNode' );
  var PNode = require( 'PNode' );
  var PDimension = require( 'PDimension' );

//Class declaration for MakeTheEquationNode

  /**
   * Constructor
   * @param challenge the challenge
   * @param model the game model
   * @param challengeSize dimensions of the view rectangle that is available for rendering the challenge
   * @param audioPlayer the audio player, for providing audio feedback during game play
   */
  /**
   * Constructor for MakeTheEquationNode
   * @param {MakeTheEquation} challenge
   * @param {LineGameModel} model
   * @param {PDimension} challengeSize
   * @param {GameAudioPlayer} audioPlayer
   */
  function MakeTheEquationNode( challenge, model, challengeSize, audioPlayer ) {
    super( challenge, model, challengeSize, audioPlayer );
    var titleNode = new PhetPText( this.title, this.TITLE_FONT, this.TITLE_COLOR );
    var maxTitleWidth = 0.45 * challengeSize.getWidth();
    if ( titleNode.getFullBoundsReference().getWidth() > maxTitleWidth ) {
      titleNode.scale( maxTitleWidth / titleNode.getFullBoundsReference().getWidth() );
    }
    var boxSize = new PDimension( 0.4 * challengeSize.getWidth(), 0.3 * challengeSize.getHeight() );
    var answerBoxNode = new EquationBoxNode( this.A_CORRECT_EQUATION, this.color, boxSize, this.createEquationNode( this.equationForm, this.answer, this.STATIC_EQUATION_FONT, this.color ) );
    answerBoxNode.setVisible( false );
    var guessBoxNode = new EquationBoxNode( this.YOUR_EQUATION, this.color, boxSize, this.createInteractiveEquationNode( this.equationForm, this.manipulationMode, this.guess, this.graph, this.INTERACTIVE_EQUATION_FONT, this.STATIC_EQUATION_FONT, this.color ) );
    var graphNode = new AnswerGraphNode( challenge );
    subclassParent.addChild( titleNode );
    subclassParent.addChild( graphNode );
    subclassParent.addChild( answerBoxNode );
    subclassParent.addChild( guessBoxNode );
    {
      guessBoxNode.setOffset( (0.5 * challengeSize.getWidth()) - (guessBoxNode.getFullBoundsReference().getWidth()) - 50, this.mvt.modelToViewY( 0 ) - guessBoxNode.getFullBoundsReference().getHeight() - 10 );
      answerBoxNode.setOffset( guessBoxNode.getXOffset(), this.mvt.modelToViewY( 0 ) + 10 );
      faceNode.setOffset( answerBoxNode.getFullBoundsReference().getCenterX() - (faceNode.getFullBoundsReference().getWidth() / 2), checkButton.getFullBoundsReference().getMaxY() - faceNode.getFullBoundsReference().getHeight() );
      titleNode.setOffset( guessBoxNode.getFullBoundsReference().getMinX(), guessBoxNode.getFullBoundsReference().getMinY() - titleNode.getFullBoundsReference().getHeight() - 20 );
    }
    if ( PhetApplication.getInstance().isDeveloperControlsEnabled() ) {
      var devAnswerNode = this.createEquationNode( this.equationForm, this.answer, this.STATIC_EQUATION_FONT, new Color( 0, 0, 0, 25 ) );
      devAnswerNode.setOffset( answerBoxNode.getFullBoundsReference().getMinX() + 30, answerBoxNode.getFullBoundsReference().getCenterY() - (devAnswerNode.getFullBoundsReference().getHeight() / 2) );
      this.addChild( devAnswerNode );
      devAnswerNode.moveToBack();
    }
    var updateIcons = new VoidFunction0()
    {

      apply: function() {
        answerBoxNode.setCorrectIconVisible( this.state.get() == this.NEXT );
        guessBoxNode.setCorrectIconVisible( this.state.get() == this.NEXT && challenge.isCorrect() );
        guessBoxNode.setIncorrectIconVisible( this.state.get() == this.NEXT && !challenge.isCorrect() );
      }
    }
    ;
    this.guess.addObserver( new SimpleObserver()
    {

      update: function() {
        updateIcons.apply();
      }
    }
  )
    ;
    this.state.addObserver( new VoidFunction1()
    {

      apply: function( state ) {
        guessBoxNode.setPickable( state == this.FIRST_CHECK || state == this.SECOND_CHECK || (state == this.NEXT && !challenge.isCorrect()) );
        guessBoxNode.setChildrenPickable( guessBoxNode.getPickable() );
        graphNode.setGuessVisible( state == this.NEXT );
        if ( state == this.NEXT && !challenge.isCorrect() ) {
          answerBoxNode.setVisible( true );
          graphNode.setAnswerPointVisible( true );
          graphNode.setGuessPointVisible( true );
          graphNode.setSlopeToolVisible( true );
        }
        updateIcons.apply();
      }
    }
  )
    ;
  }

  return inherit( ChallengeNode, MakeTheEquationNode, {

    createInteractiveEquationNode: function( equationForm, manipulationMode, line, graph, interactiveFont, staticFont, staticColor ) {
      if ( equationForm == this.SLOPE_INTERCEPT ) {
        var interactiveSlope = (manipulationMode == this.SLOPE) || (manipulationMode == this.SLOPE_INTERCEPT);
        var interactiveIntercept = (manipulationMode == this.INTERCEPT) || (manipulationMode == this.SLOPE_INTERCEPT);
        return new SlopeInterceptEquationNode( line, new Property( new DoubleRange( this.yRange ) ), new Property( new DoubleRange( this.xRange ) ), new Property( new DoubleRange( this.yRange ) ), interactiveSlope, interactiveIntercept, interactiveFont, staticFont, staticColor );
      }
      else if ( equationForm == this.POINT_SLOPE ) {
        var interactivePoint = (manipulationMode == this.POINT) || (manipulationMode == this.POINT_SLOPE);
        var interactiveSlope = (manipulationMode == this.SLOPE) || (manipulationMode == this.POINT_SLOPE);
        return new PointSlopeEquationNode( line, new Property( new DoubleRange( this.xRange ) ), new Property( new DoubleRange( this.yRange ) ), new Property( new DoubleRange( this.yRange ) ), new Property( new DoubleRange( this.xRange ) ), interactivePoint, interactivePoint, interactiveSlope, interactiveFont, staticFont, staticColor );
      }
      else {
        throw new IllegalArgumentException( "unsupported equation form: " + equationForm );
      }
    },


  } );

} );

