// Copyright 2002-2013, University of Colorado Boulder

define( function( require ) {
  'use strict';

  var inherit = require( 'PHET_CORE/inherit' );

  var Color = require( 'Color' );
  var Font = require( 'Font' );
  var MessageFormat = require( 'MessageFormat' );
  var MathUtil = require( 'MathUtil' );
  var Property = require( 'Property' );
  var DoubleRange = require( 'DoubleRange' );
  var RichSimpleObserver = require( 'RichSimpleObserver' );
  var VoidFunction1 = require( 'VoidFunction1' );
  var PhetFont = require( 'PhetFont' );
  var HTMLNode = require( 'HTMLNode' );
  var PhetPText = require( 'PhetPText' );
  var ZeroOffsetNode = require( 'ZeroOffsetNode' );
  var LGColors = require( 'LGColors' );
  var Strings = require( 'Strings' );
  var UserComponents = require( 'UserComponents' );
  var Line = require( 'Line' );
  var DynamicValueNode = require( 'DynamicValueNode' );
  var EquationNode = require( 'EquationNode' );
  var MinusNode = require( 'MinusNode' );
  var PlusNode = require( 'PlusNode' );
  var SlopeUndefinedNode = require( 'SlopeUndefinedNode' );
  var UndefinedSlopeIndicator = require( 'UndefinedSlopeIndicator' );
  var RiseSpinnerNode = require( 'RiseSpinnerNode' );
  var RunSpinnerNode = require( 'RunSpinnerNode' );
  var SpinnerNode = require( 'SpinnerNode' );
  var SlopeColors = require( 'SlopeColors' );
  var X1Y1Colors = require( 'X1Y1Colors' );
  var SlopeInterceptEquationNode = require( 'SlopeInterceptEquationNode' );
  var PNode = require( 'PNode' );
  var PPath = require( 'PPath' );

//Class declaration for PointSlopeEquationNode

  /**
   * Constructor for PointSlopeEquationNode
   * @param {Line} line
   * @param {PhetFont} font
   * @param {Color} color
   */
  function PointSlopeEquationNode( line, font, color ) {
    this( new Property( line ), new Property( new DoubleRange( 0, 1 ) ), new Property( new DoubleRange( 0, 1 ) ), new Property( new DoubleRange( 0, 1 ) ), new Property( new DoubleRange( 0, 1 ) ), false, false, false, font, font, color );
  }

  /**
   * Constructor for PointSlopeEquationNode
   * @param {Property<Line>} interactiveLine
   * @param {Property<DoubleRange>} x1Range
   * @param {Property<DoubleRange>} y1Range
   * @param {Property<DoubleRange>} riseRange
   * @param {Property<DoubleRange>} runRange
   * @param {boolean} interactiveX1
   * @param {boolean} interactiveY1
   * @param {boolean} interactiveSlope
   * @param {PhetFont} interactiveFont
   * @param {PhetFont} staticFont
   * @param {Color} staticColor
   */
  function PointSlopeEquationNode( interactiveLine, x1Range, y1Range, riseRange, runRange, interactiveX1, interactiveY1, interactiveSlope, interactiveFont, staticFont, staticColor ) {
    super( staticFont.getSize() );
    this.x1 = new Property( this.x1 );
    this.y1 = new Property( this.y1 );
    this.rise = new Property( this.rise );
    this.run = new Property( this.run );
    var maxSlopeSpinnerWidth = this.computeMaxSlopeSpinnerWidth( riseRange, runRange, interactiveFont, FORMAT );
    yLeftParenNode = new PhetPText( "(", staticFont, staticColor );
    yNode = new PhetPText( this.SYMBOL_Y, staticFont, staticColor );
    yOperatorNode = new PNode();
    if ( interactiveY1 ) {
      y1Node = new ZeroOffsetNode( new SpinnerNode( this.y1Spinner, y1, y1Range, new X1Y1Colors(), interactiveFont, FORMAT ) );
    }
    else {
      y1Node = new DynamicValueNode( y1, FORMAT, staticFont, staticColor, true );
    }
    yRightParenNode = new PhetPText( ")", staticFont, staticColor );
    y1MinusSignNode = new MinusNode( signLineSize, staticColor );
    equalsNode = new PhetPText( "=", staticFont, staticColor );
    slopeMinusSignNode = new MinusNode( signLineSize, staticColor );
    if ( interactiveSlope ) {
      riseNode = new ZeroOffsetNode( new RiseSpinnerNode( this.riseSpinner, rise, run, riseRange, new SlopeColors(), interactiveFont, FORMAT ) );
      runNode = new ZeroOffsetNode( new RunSpinnerNode( this.runSpinner, rise, run, runRange, new SlopeColors(), interactiveFont, FORMAT ) );
    }
    else {
      riseNode = new DynamicValueNode( rise, FORMAT, staticFont, staticColor, true );
      runNode = new DynamicValueNode( run, FORMAT, staticFont, staticColor, true );
    }
    fractionLineNode = new PPath( this.createFractionLineShape( maxSlopeSpinnerWidth ) )
    {

      {
        this.setStroke( null );
        this.setPaint( staticColor );
      }
    }
    ;
    xLeftParenNode = new PhetPText( "(", staticFont, staticColor );
    xNode = new PhetPText( this.SYMBOL_X, staticFont, staticColor );
    xOperatorNode = new PNode();
    if ( interactiveX1 ) {
      x1Node = new ZeroOffsetNode( new SpinnerNode( this.x1Spinner, x1, x1Range, new X1Y1Colors(), interactiveFont, FORMAT ) );
    }
    else {
      x1Node = new DynamicValueNode( x1, FORMAT, staticFont, staticColor, true );
    }
    xRightParenNode = new PhetPText( ")", staticFont, staticColor );
    var lineUpdater = new RichSimpleObserver()
    {

      update: function() {
        if ( !updatingControls ) {
          interactiveLine.set( Line.createPointSlope( x1.get(), y1.get(), rise.get(), run.get(), this.color ) );
        }
      }
    }
    ;
    lineUpdater.observe( rise, run, x1, y1 );
    interactiveLine.addObserver( new VoidFunction1()
    {

      apply: function( line ) {
        updatingControls = true;
        {
          x1.set( this.x1 );
          y1.set( this.y1 );
          rise.set( interactiveSlope ? this.rise : line.getSimplifiedRise() );
          run.set( interactiveSlope ? this.run : line.getSimplifiedRun() );
        }
        updatingControls = false;
        this.updateLayout( line, interactiveX1, interactiveY1, interactiveSlope, staticFont, staticColor );
      }
    }
  )
    ;
  }

  return inherit( EquationNode, PointSlopeEquationNode, {

    updateLayout: function( line, interactiveX1, interactiveY1, interactiveSlope, staticFont, staticColor ) {
      var interactive = interactiveX1 || interactiveY1 || interactiveSlope;
      this.removeAllChildren();
      xOperatorNode.removeAllChildren();
      yOperatorNode.removeAllChildren();
      if ( line.undefinedSlope() && !interactive ) {
        this.addChild( new SlopeUndefinedNode( line, staticFont, staticColor ) );
        return;
      }
      else if ( (line.same( this.Y_EQUALS_X_LINE ) || line.same( this.Y_EQUALS_NEGATIVE_X_LINE )) && !interactive ) {
        this.addChild( new SlopeInterceptEquationNode( Line.createSlopeIntercept( this.rise, this.run, 0, this.color ), staticFont, staticColor ) );
        return;
      }
      {
        if ( interactiveX1 || this.x1 >= 0 ) {
          xOperatorNode.addChild( new MinusNode( operatorLineSize, staticColor ) );
        }
        else {
          xOperatorNode.addChild( new PlusNode( operatorLineSize, staticColor ) );
        }
        if ( interactiveY1 || this.y1 >= 0 ) {
          yOperatorNode.addChild( new MinusNode( operatorLineSize, staticColor ) );
        }
        else {
          yOperatorNode.addChild( new PlusNode( operatorLineSize, staticColor ) );
        }
      }
      if ( this.rise == 0 && !interactiveSlope && !interactiveX1 ) {
        this.addChild( yNode );
        this.addChild( equalsNode );
        this.addChild( y1Node );
        yNode.setOffset( 0, 0 );
        equalsNode.setOffset( yNode.getFullBoundsReference().getMaxX() + relationalOperatorXSpacing, yNode.getYOffset() );
        if ( interactiveY1 || this.y1 >= 0 ) {
          y1Node.setOffset( equalsNode.getFullBoundsReference().getMaxX() + relationalOperatorXSpacing, yNode.getFullBoundsReference().getCenterY() - (y1Node.getFullBoundsReference().getHeight() / 2) );
        }
        else {
          this.addChild( y1MinusSignNode );
          y1MinusSignNode.setOffset( equalsNode.getFullBoundsReference().getMaxX() + relationalOperatorXSpacing, equalsNode.getFullBoundsReference().getCenterY() - (y1MinusSignNode.getFullBoundsReference().getHeight() / 2) + operatorYFudgeFactor );
          y1Node.setOffset( y1MinusSignNode.getFullBoundsReference().getMaxX() + integerSignXSpacing, yNode.getFullBoundsReference().getCenterY() - (y1Node.getFullBoundsReference().getHeight() / 2) );
        }
      }
      else {
        var previousNode;
        this.addChild( yLeftParenNode );
        this.addChild( yNode );
        this.addChild( yOperatorNode );
        this.addChild( y1Node );
        this.addChild( yRightParenNode );
        yLeftParenNode.setOffset( 0, 0 );
        yNode.setOffset( yLeftParenNode.getFullBoundsReference().getMaxX() + parenXSpacing, yLeftParenNode.getYOffset() );
        yOperatorNode.setOffset( yNode.getFullBoundsReference().getMaxX() + operatorXSpacing, equalsNode.getFullBoundsReference().getCenterY() - (yOperatorNode.getFullBoundsReference().getHeight() / 2) + operatorYFudgeFactor );
        y1Node.setOffset( yOperatorNode.getFullBoundsReference().getMaxX() + operatorXSpacing, yNode.getFullBoundsReference().getCenterY() - (y1Node.getFullBoundsReference().getHeight() / 2) );
        yRightParenNode.setOffset( y1Node.getFullBoundsReference().getMaxX() + parenXSpacing, yNode.getYOffset() );
        this.addChild( equalsNode );
        equalsNode.setOffset( yRightParenNode.getFullBoundsReference().getMaxX() + relationalOperatorXSpacing, yNode.getYOffset() );
        var previousXOffset;
        if ( interactiveSlope ) {
          this.addChild( riseNode );
          this.addChild( fractionLineNode );
          this.addChild( runNode );
          fractionLineNode.setOffset( equalsNode.getFullBoundsReference().getMaxX() + relationalOperatorXSpacing, equalsNode.getFullBoundsReference().getCenterY() + 2 );
          riseNode.setOffset( fractionLineNode.getFullBoundsReference().getCenterX() - (riseNode.getFullBoundsReference().getWidth() / 2), fractionLineNode.getFullBoundsReference().getMinY() - riseNode.getFullBoundsReference().getHeight() - spinnersYSpacing );
          runNode.setOffset( fractionLineNode.getFullBoundsReference().getCenterX() - (runNode.getFullBoundsReference().getWidth() / 2), fractionLineNode.getFullBoundsReference().getMinY() + spinnersYSpacing );
          previousNode = fractionLineNode;
          previousXOffset = fractionalSlopeXSpacing;
        }
        else {
          var slope = line.getSlope();
          var zeroSlope = (slope == 0);
          var unitySlope = (Math.abs( slope ) == 1);
          var integerSlope = MathUtil.isInteger( slope );
          var positiveSlope = (slope > 0);
          var fractionalSlope = (!zeroSlope && !unitySlope && !integerSlope);
          var lineWidth = Math.max( riseNode.getFullBoundsReference().getWidth(), runNode.getFullBoundsReference().getWidth() );
          fractionLineNode.setPathTo( this.createFractionLineShape( lineWidth ) );
          if ( positiveSlope || zeroSlope ) {
            previousNode = equalsNode;
            previousXOffset = relationalOperatorXSpacing;
          }
          else {
            this.addChild( slopeMinusSignNode );
            slopeMinusSignNode.setOffset( equalsNode.getFullBoundsReference().getMaxX() + relationalOperatorXSpacing, equalsNode.getFullBoundsReference().getCenterY() - (slopeMinusSignNode.getFullBoundsReference().getHeight() / 2) + slopeSignYFudgeFactor + slopeSignYOffset );
            previousNode = slopeMinusSignNode;
            previousXOffset = (fractionalSlope ? fractionSignXSpacing : integerSignXSpacing);
          }
          if ( line.undefinedSlope() || fractionalSlope ) {
            this.addChild( riseNode );
            this.addChild( fractionLineNode );
            this.addChild( runNode );
            fractionLineNode.setOffset( previousNode.getFullBoundsReference().getMaxX() + previousXOffset, equalsNode.getFullBoundsReference().getCenterY() + 2 );
            riseNode.setOffset( fractionLineNode.getFullBoundsReference().getCenterX() - (riseNode.getFullBoundsReference().getWidth() / 2), fractionLineNode.getFullBoundsReference().getMinY() - riseNode.getFullBoundsReference().getHeight() - ySpacing );
            runNode.setOffset( fractionLineNode.getFullBoundsReference().getCenterX() - (runNode.getFullBoundsReference().getWidth() / 2), fractionLineNode.getFullBoundsReference().getMinY() + ySpacing );
            previousNode = fractionLineNode;
            previousXOffset = fractionalSlopeXSpacing;
          }
          else if ( zeroSlope ) {
            this.addChild( riseNode );
            riseNode.setOffset( equalsNode.getFullBoundsReference().getMaxX() + relationalOperatorXSpacing, yNode.getYOffset() );
            previousNode = riseNode;
            previousXOffset = integerSlopeXSpacing;
          }
          else if ( unitySlope ) {
            previousXOffset = relationalOperatorXSpacing;
          }
          else if ( integerSlope ) {
            this.addChild( riseNode );
            riseNode.setOffset( previousNode.getFullBoundsReference().getMaxX() + previousXOffset, yNode.getYOffset() );
            previousNode = riseNode;
            previousXOffset = integerSlopeXSpacing;
          }
          else {
            throw new IllegalStateException( "programming error, didn't handle some slope case" );
          }
        }
        if ( interactiveX1 || interactiveSlope || this.rise != 0 ) {
          this.addChild( xLeftParenNode );
          this.addChild( xNode );
          this.addChild( xOperatorNode );
          this.addChild( x1Node );
          this.addChild( xRightParenNode );
          xLeftParenNode.setOffset( previousNode.getFullBoundsReference().getMaxX() + previousXOffset, yNode.getYOffset() );
          xNode.setOffset( xLeftParenNode.getFullBoundsReference().getMaxX() + parenXSpacing, yNode.getYOffset() );
          xOperatorNode.setOffset( xNode.getFullBoundsReference().getMaxX() + operatorXSpacing, equalsNode.getFullBoundsReference().getCenterY() - (xOperatorNode.getFullBoundsReference().getHeight() / 2) + operatorYFudgeFactor );
          x1Node.setOffset( xOperatorNode.getFullBoundsReference().getMaxX() + operatorXSpacing, xNode.getFullBoundsReference().getCenterY() - (x1Node.getFullBoundsReference().getHeight() / 2) );
          xRightParenNode.setOffset( x1Node.getFullBoundsReference().getMaxX() + parenXSpacing, yNode.getYOffset() );
        }
        else if ( this.rise == 0 ) {
        }
        else {
          throw new IllegalStateException( "programming error, didn't handle some x-term case" );
        }
      }
      if ( line.undefinedSlope() ) {
        var undefinedSlopeIndicator = new UndefinedSlopeIndicator( this.getFullBoundsReference().getWidth(), this.getFullBoundsReference().getHeight() );
        undefinedSlopeIndicator.setOffset( 0, fractionLineNode.getFullBoundsReference().getCenterY() - (undefinedSlopeIndicator.getFullBoundsReference().getHeight() / 2) + undefinedSlopeYFudgeFactor );
        this.addChild( undefinedSlopeIndicator );
      }
    },


    createGeneralFormNode: function() {
      var html = MessageFormat.format( "<html>({0} - {1}<font size='-1'><sub>1</sub></font>) = {2}({3} - {4}<font size='-1'><sub>1</sub></font>)</html>", this.SYMBOL_Y, this.SYMBOL_Y, this.SYMBOL_SLOPE, this.SYMBOL_X, this.SYMBOL_X );
      return new HTMLNode( html, this.INTERACTIVE_LINE, new PhetFont( this.BOLD, 18 ) );
    },


  } );

} );

