// Copyright 2002-2013, University of Colorado Boulder

define( function( require ) {
  'use strict';

  var inherit = require( 'PHET_CORE/inherit' );

  var Color = require( 'Color' );
  var Font = require( 'Font' );
  var Rectangle2D = require( 'Rectangle2D' );
  var MessageFormat = require( 'MessageFormat' );
  var NumberFormat = require( 'NumberFormat' );
  var MathUtil = require( 'MathUtil' );
  var Property = require( 'Property' );
  var DefaultDecimalFormat = require( 'DefaultDecimalFormat' );
  var DoubleRange = require( 'DoubleRange' );
  var RichSimpleObserver = require( 'RichSimpleObserver' );
  var VoidFunction1 = require( 'VoidFunction1' );
  var PhetFont = require( 'PhetFont' );
  var HTMLNode = require( 'HTMLNode' );
  var PhetPPath = require( 'PhetPPath' );
  var PhetPText = require( 'PhetPText' );
  var ZeroOffsetNode = require( 'ZeroOffsetNode' );
  var LGColors = require( 'LGColors' );
  var Strings = require( 'Strings' );
  var UserComponents = require( 'UserComponents' );
  var Line = require( 'Line' );
  var EquationNode = require( 'EquationNode' );
  var MinusNode = require( 'MinusNode' );
  var NumberBackgroundNode = require( 'NumberBackgroundNode' );
  var UndefinedSlopeIndicator = require( 'UndefinedSlopeIndicator' );
  var CoordinateSpinnerNode = require( 'CoordinateSpinnerNode' );
  var X1Y1Colors = require( 'X1Y1Colors' );
  var X2Y2Colors = require( 'X2Y2Colors' );
  var PNode = require( 'PNode' );
  var PPath = require( 'PPath' );
  var PText = require( 'PText' );

//Class declaration for SlopeEquationNode

  /**
   * Constructor for SlopeEquationNode
   * @param {Property<Line>} interactiveLine
   * @param {Property<DoubleRange>} xRange
   * @param {Property<DoubleRange>} yRange
   * @param {PhetFont} interactiveFont
   * @param {PhetFont} staticFont
   * @param {Color} staticColor
   */
  function SlopeEquationNode( interactiveLine, xRange, yRange, interactiveFont, staticFont, staticColor ) {
    super( staticFont.getSize() );
    var x1 = new Property( this.x1 );
    var y1 = new Property( this.y1 );
    var x2 = new Property( this.x2 );
    var y2 = new Property( this.y2 );
    var mNode = new PhetPText( this.SYMBOL_SLOPE, staticFont, staticColor );
    var interactiveEqualsNode = new PhetPText( "=", staticFont, staticColor );
    var y2Node = new ZeroOffsetNode( new CoordinateSpinnerNode( this.y2Spinner, y2, x2, y1, x1, yRange, new X2Y2Colors(), interactiveFont, FORMAT ) );
    var numeratorOperatorNode = new PhetPText( "-", staticFont, staticColor );
    var y1Node = new ZeroOffsetNode( new CoordinateSpinnerNode( this.y1Spinner, y1, x1, y2, x2, yRange, new X1Y1Colors(), interactiveFont, FORMAT ) );
    var interactiveFractionLineNode = new PhetPPath( this.createFractionLineShape( 10 ), staticColor, null, null );
    var x2Node = new ZeroOffsetNode( new CoordinateSpinnerNode( this.x2Spinner, x2, y2, x1, y1, xRange, new X2Y2Colors(), interactiveFont, FORMAT ) );
    var denominatorOperatorNode = new PhetPText( "-", staticFont, staticColor );
    var x1Node = new ZeroOffsetNode( new CoordinateSpinnerNode( this.x1Spinner, x1, y1, x2, y2, xRange, new X1Y1Colors(), interactiveFont, FORMAT ) );
    var unsimplifiedEqualsNode = new PhetPText( "=", staticFont, staticColor );
    unsimplifiedRiseNode = new PNode();
    unsimplifiedRunNode = new PNode();
    var unsimplifiedFractionLineNode = new PhetPPath( this.createFractionLineShape( 10 ), staticColor, null, null );
    var maxRangeLength = Math.max( xRange.get().getLength(), yRange.get().getLength() );
    var maxUnsimplifiedWidth = new PhetPText( FORMAT.format( -maxRangeLength ), interactiveFont, staticColor ).getFullBoundsReference().getWidth();
    var parentNode = new PNode();
    this.addChild( parentNode );
    {
      parentNode.addChild( mNode );
      parentNode.addChild( interactiveEqualsNode );
      parentNode.addChild( y2Node );
      parentNode.addChild( numeratorOperatorNode );
      parentNode.addChild( y1Node );
      parentNode.addChild( interactiveFractionLineNode );
      parentNode.addChild( x2Node );
      parentNode.addChild( denominatorOperatorNode );
      parentNode.addChild( x1Node );
      parentNode.addChild( unsimplifiedEqualsNode );
      parentNode.addChild( unsimplifiedRiseNode );
      parentNode.addChild( unsimplifiedFractionLineNode );
      parentNode.addChild( unsimplifiedRunNode );
    }
    {
      mNode.setOffset( 0, 0 );
      interactiveEqualsNode.setOffset( mNode.getFullBoundsReference().getMaxX() + relationalOperatorXSpacing, mNode.getYOffset() );
      interactiveFractionLineNode.setOffset( interactiveEqualsNode.getFullBoundsReference().getMaxX() + relationalOperatorXSpacing, interactiveEqualsNode.getFullBoundsReference().getCenterY() + fractionLineYFudgeFactor );
      y2Node.setOffset( interactiveFractionLineNode.getXOffset(), interactiveFractionLineNode.getFullBoundsReference().getMinY() - y2Node.getFullBoundsReference().getHeight() - spinnersYSpacing );
      numeratorOperatorNode.setOffset( y2Node.getFullBoundsReference().getMaxX() + operatorXSpacing, y2Node.getFullBoundsReference().getCenterY() - (numeratorOperatorNode.getFullBoundsReference().getHeight() / 2) );
      y1Node.setOffset( numeratorOperatorNode.getFullBoundsReference().getMaxX() + operatorXSpacing, y2Node.getYOffset() );
      var leftLineLength = y1Node.getFullBoundsReference().getMaxX() - y2Node.getFullBoundsReference().getMinX();
      interactiveFractionLineNode.setPathTo( this.createFractionLineShape( leftLineLength ) );
      x2Node.setOffset( y2Node.getXOffset(), interactiveFractionLineNode.getFullBoundsReference().getMaxY() + spinnersYSpacing );
      denominatorOperatorNode.setOffset( x2Node.getFullBoundsReference().getMaxX() + operatorXSpacing, x2Node.getFullBoundsReference().getCenterY() - (denominatorOperatorNode.getFullBoundsReference().getHeight() / 2) );
      x1Node.setOffset( denominatorOperatorNode.getFullBoundsReference().getMaxX() + operatorXSpacing, x2Node.getYOffset() );
      unsimplifiedEqualsNode.setOffset( interactiveFractionLineNode.getFullBoundsReference().getMaxX() + relationalOperatorXSpacing, interactiveEqualsNode.getYOffset() );
      unsimplifiedFractionLineNode.setOffset( unsimplifiedEqualsNode.getFullBoundsReference().getMaxX() + relationalOperatorXSpacing, interactiveFractionLineNode.getYOffset() );
    }
    var updateLayout = new VoidFunction1()
    {

      apply: function( line ) {
        unsimplifiedRiseNode.setOffset( unsimplifiedFractionLineNode.getFullBoundsReference().getCenterX() - (unsimplifiedRiseNode.getFullBoundsReference().getWidth() / 2), unsimplifiedFractionLineNode.getFullBoundsReference().getMinY() - unsimplifiedRiseNode.getFullBoundsReference().getHeight() - slopeYSpacing );
        unsimplifiedRunNode.setOffset( unsimplifiedFractionLineNode.getFullBoundsReference().getCenterX() - (unsimplifiedRunNode.getFullBoundsReference().getWidth() / 2), unsimplifiedFractionLineNode.getFullBoundsReference().getMaxY() + slopeYSpacing );
      }
    }
    ;
    var lineUpdater = new RichSimpleObserver()
    {

      update: function() {
        if ( !updatingControls ) {
          interactiveLine.set( new Line( x1.get(), y1.get(), x2.get(), y2.get(), this.color ) );
        }
      }
    }
    ;
    lineUpdater.observe( x1, y1, x2, y2 );
    interactiveLine.addObserver( new VoidFunction1()
    {

      PNode
      undefinedSlopeIndicator;

      apply: function( line ) {
        updatingControls = true;
        {
          x1.set( this.x1 );
          y1.set( this.y1 );
          x2.set( this.x2 );
          y2.set( this.y2 );
        }
        updatingControls = false;
        {
          var margin = 3;
          var cornerRadius = 10;
          this.removeChild( unsimplifiedRiseNode );
          unsimplifiedRiseNode = new NumberBackgroundNode( this.rise, FORMAT, staticFont, staticColor, this.SLOPE, margin, margin, cornerRadius, maxUnsimplifiedWidth );
          this.addChild( unsimplifiedRiseNode );
          this.removeChild( unsimplifiedRunNode );
          unsimplifiedRunNode = new NumberBackgroundNode( this.run, FORMAT, staticFont, staticColor, this.SLOPE, margin, margin, cornerRadius, maxUnsimplifiedWidth );
          this.addChild( unsimplifiedRunNode );
          unsimplifiedFractionLineNode.setPathTo( this.createFractionLineShape( Math.max( unsimplifiedRiseNode.getFullBoundsReference().getWidth(), unsimplifiedRunNode.getFullBoundsReference().getWidth() ) ) );
        }
        updateLayout.apply( line );
        if ( undefinedSlopeIndicator != null ) {
          this.removeChild( undefinedSlopeIndicator );
        }
        if ( line.undefinedSlope() ) {
          undefinedSlopeIndicator = new UndefinedSlopeIndicator( this.getFullBoundsReference().getWidth(), this.getFullBoundsReference().getHeight() );
          undefinedSlopeIndicator.setOffset( parentNode.getXOffset(), parentNode.getFullBoundsReference().getCenterY() - (undefinedSlopeIndicator.getFullBoundsReference().getHeight() / 2) + undefinedSlopeYFudgeFactor );
          this.addChild( undefinedSlopeIndicator );
        }
      }
    }
  )
    ;
  }

  /**
   * Constructor for SlopeEquationNode
   * @param {Property<Line>} line
   * @param {PhetFont} font
   * @param {Color} color
   */
  function SlopeEquationNode( line, font, color ) {
    super( font.getSize() );
    var slopeIsNode = new PhetPText( MessageFormat.format( this.SLOPE_IS, this.SYMBOL_SLOPE ), font, color );
    var minusSignNode = new MinusNode( signLineSize, color );
    var riseNode = new PhetPText( "?", font, color );
    var runNode = new PhetPText( "?", font, color );
    var fractionLineNode = new PhetPPath( this.createFractionLineShape( 1 ), color, null, null );
    this.addChild( slopeIsNode );
    slopeIsNode.setOffset( 0, 0 );
    var updateLayout = new VoidFunction1()
    {

      apply: function( line ) {
        this.removeChild( minusSignNode );
        this.removeChild( riseNode );
        this.removeChild( runNode );
        this.removeChild( fractionLineNode );
        if ( line.undefinedSlope() ) {
          this.addChild( riseNode );
          riseNode.setText( this.UNDEFINED );
          riseNode.setOffset( slopeIsNode.getFullBoundsReference().getMaxX() + relationalOperatorXSpacing, slopeIsNode.getY() );
        }
        else if ( line.getSlope() == 0 ) {
          this.addChild( riseNode );
          riseNode.setText( "0" );
          riseNode.setOffset( slopeIsNode.getFullBoundsReference().getMaxX() + relationalOperatorXSpacing, slopeIsNode.getY() );
        }
        else {
          var nextXOffset;
          if ( line.getSlope() < 0 ) {
            this.addChild( minusSignNode );
            minusSignNode.setOffset( slopeIsNode.getFullBoundsReference().getMaxX() + relationalOperatorXSpacing, slopeIsNode.getFullBoundsReference().getCenterY() - (minusSignNode.getFullBoundsReference().getHeight() / 2) + slopeSignYFudgeFactor + slopeSignYOffset );
            nextXOffset = minusSignNode.getFullBoundsReference().getMaxX() + operatorXSpacing;
          }
          else {
            nextXOffset = slopeIsNode.getFullBoundsReference().getMaxX() + relationalOperatorXSpacing;
          }
          if ( MathUtil.isInteger( line.getSlope() ) ) {
            this.addChild( riseNode );
            riseNode.setText( FORMAT.format( Math.abs( line.getSlope() ) ) );
            riseNode.setOffset( nextXOffset, slopeIsNode.getYOffset() );
          }
          else {
            this.addChild( fractionLineNode );
            this.addChild( riseNode );
            this.addChild( runNode );
            riseNode.setText( FORMAT.format( Math.abs( line.getSimplifiedRise() ) ) );
            runNode.setText( FORMAT.format( Math.abs( line.getSimplifiedRun() ) ) );
            fractionLineNode.setPathTo( this.createFractionLineShape( Math.max( riseNode.getFullBoundsReference().getWidth(), runNode.getFullBoundsReference().getWidth() ) ) );
            fractionLineNode.setOffset( nextXOffset, slopeIsNode.getFullBoundsReference().getCenterY() + fractionLineYFudgeFactor );
            riseNode.setOffset( fractionLineNode.getFullBoundsReference().getCenterX() - (riseNode.getFullBoundsReference().getWidth() / 2), fractionLineNode.getFullBoundsReference().getMinY() - riseNode.getFullBoundsReference().getHeight() - ySpacing );
            runNode.setOffset( fractionLineNode.getFullBoundsReference().getCenterX() - (runNode.getFullBoundsReference().getWidth() / 2), fractionLineNode.getFullBoundsReference().getMaxY() + ySpacing );
          }
        }
      }
    }
    ;
    line.addObserver( updateLayout );
  }

  /**
   * Constructor for SlopeEquationNode
   * @param {Line} line
   * @param {PhetFont} font
   * @param {Color} color
   */
  function SlopeEquationNode( line, font, color ) {
    this( new Property( line ), font, color );
  }

  return inherit( EquationNode, SlopeEquationNode, {

    createGeneralFormNode: function() {
      var color = this.INTERACTIVE_LINE;
      var font = new PhetFont( this.BOLD, 18 );
      var htmlLeftSide = MessageFormat.format( "{0}&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;{1} =", this.SLOPE, this.SYMBOL_SLOPE );
      var leftSideNode = new HTMLNode( htmlLeftSide, color, font );
      var pattern = "<html>{0}<font size='-1'><sub>2</sub></font> - {1}<font size='-1'><sub>1</sub></font></html>";
      var htmlNumerator = MessageFormat.format( pattern, this.SYMBOL_Y, this.SYMBOL_Y );
      var numeratorNode = new HTMLNode( htmlNumerator, color, font );
      var htmlDenominator = MessageFormat.format( pattern, this.SYMBOL_X, this.SYMBOL_X );
      var denominatorNode = new HTMLNode( htmlDenominator, color, font );
      var length = Math.max( numeratorNode.getFullBoundsReference().getWidth(), denominatorNode.getFullBoundsReference().getWidth() );
      var fractionLineNode = new PPath( new Rectangle2D.Double( 0, 0, length, 1 ) );
      fractionLineNode.setPaint( color );
      fractionLineNode.setStroke( null );
      var parentNode = new PNode();
      parentNode.addChild( leftSideNode );
      parentNode.addChild( numeratorNode );
      parentNode.addChild( denominatorNode );
      parentNode.addChild( fractionLineNode );
      leftSideNode.setOffset( 0, 0 );
      fractionLineNode.setOffset( leftSideNode.getFullBoundsReference().getMaxX() + 5, leftSideNode.getFullBoundsReference().getCenterY() - (fractionLineNode.getFullBoundsReference().getHeight() / 2) + 3 );
      numeratorNode.setOffset( fractionLineNode.getFullBoundsReference().getCenterX() - (numeratorNode.getFullBoundsReference().getWidth() / 2), fractionLineNode.getFullBoundsReference().getMinY() - numeratorNode.getFullBoundsReference().getHeight() - 1 );
      denominatorNode.setOffset( fractionLineNode.getFullBoundsReference().getCenterX() - (denominatorNode.getFullBoundsReference().getWidth() / 2), fractionLineNode.getFullBoundsReference().getMaxY() + 1 );
      return new ZeroOffsetNode( parentNode );
    },


  } );

} );

