// Copyright 2002-2013, University of Colorado Boulder

define( function( require ) {
  'use strict';

  var inherit = require( 'PHET_CORE/inherit' );

  var Color = require( 'Color' );
  var Font = require( 'Font' );
  var MessageFormat = require( 'MessageFormat' );
  var MathUtil = require( 'MathUtil' );
  var Property = require( 'Property' );
  var DoubleRange = require( 'DoubleRange' );
  var RichSimpleObserver = require( 'RichSimpleObserver' );
  var VoidFunction1 = require( 'VoidFunction1' );
  var PhetFont = require( 'PhetFont' );
  var HTMLNode = require( 'HTMLNode' );
  var PhetPText = require( 'PhetPText' );
  var ZeroOffsetNode = require( 'ZeroOffsetNode' );
  var LGColors = require( 'LGColors' );
  var Strings = require( 'Strings' );
  var UserComponents = require( 'UserComponents' );
  var Fraction = require( 'Fraction' );
  var Line = require( 'Line' );
  var DynamicValueNode = require( 'DynamicValueNode' );
  var EquationNode = require( 'EquationNode' );
  var MinusNode = require( 'MinusNode' );
  var PlusNode = require( 'PlusNode' );
  var SlopeUndefinedNode = require( 'SlopeUndefinedNode' );
  var UndefinedSlopeIndicator = require( 'UndefinedSlopeIndicator' );
  var RiseSpinnerNode = require( 'RiseSpinnerNode' );
  var RunSpinnerNode = require( 'RunSpinnerNode' );
  var SpinnerNode = require( 'SpinnerNode' );
  var InterceptColors = require( 'InterceptColors' );
  var SlopeColors = require( 'SlopeColors' );
  var PNode = require( 'PNode' );
  var PPath = require( 'PPath' );

//Class declaration for SlopeInterceptEquationNode

  /**
   * Constructor for SlopeInterceptEquationNode
   * @param {Line} line
   * @param {PhetFont} font
   * @param {Color} color
   */
  function SlopeInterceptEquationNode( line, font, color ) {
    this( new Property( line ), new Property( new DoubleRange( 0, 1 ) ), new Property( new DoubleRange( 0, 1 ) ), new Property( new DoubleRange( 0, 1 ) ), false, false, font, font, color );
  }

  /**
   * Constructor for SlopeInterceptEquationNode
   * @param {Property<Line>} interactiveLine
   * @param {Property<DoubleRange>} riseRange
   * @param {Property<DoubleRange>} runRange
   * @param {Property<DoubleRange>} yInterceptRange
   * @param {boolean} interactiveSlope
   * @param {boolean} interactiveIntercept
   * @param {PhetFont} interactiveFont
   * @param {PhetFont} staticFont
   * @param {Color} staticColor
   */
  function SlopeInterceptEquationNode( interactiveLine, riseRange, runRange, yInterceptRange, interactiveSlope, interactiveIntercept, interactiveFont, staticFont, staticColor ) {
    super( staticFont.getSize() );
    this.rise = new Property( this.rise );
    this.run = new Property( this.run );
    this.yIntercept = new Property( this.y1 );
    var fractionalIntercept = interactiveLine.get().getYIntercept();
    this.yInterceptNumerator = new Property( this.numerator );
    this.yInterceptDenominator = new Property( this.denominator );
    var maxSlopeSpinnerWidth = this.computeMaxSlopeSpinnerWidth( riseRange, runRange, interactiveFont, FORMAT );
    yNode = new PhetPText( this.SYMBOL_Y, staticFont, staticColor );
    equalsNode = new PhetPText( "=", staticFont, staticColor );
    slopeMinusSignNode = new MinusNode( signLineSize, staticColor );
    if ( interactiveSlope ) {
      riseNode = new ZeroOffsetNode( new RiseSpinnerNode( this.riseSpinner, rise, run, riseRange, new SlopeColors(), interactiveFont, FORMAT ) );
      runNode = new ZeroOffsetNode( new RunSpinnerNode( this.runSpinner, rise, run, runRange, new SlopeColors(), interactiveFont, FORMAT ) );
    }
    else {
      riseNode = new DynamicValueNode( rise, FORMAT, staticFont, staticColor, true );
      runNode = new DynamicValueNode( run, FORMAT, staticFont, staticColor, true );
    }
    slopeFractionLineNode = new PPath( this.createFractionLineShape( maxSlopeSpinnerWidth ) )
    {

      {
        this.setStroke( null );
        this.setPaint( staticColor );
      }
    }
    ;
    xNode = new PhetPText( this.SYMBOL_X, staticFont, staticColor );
    operatorNode = new PNode();
    yInterceptMinusSignNode = new MinusNode( signLineSize, staticColor );
    yInterceptNode = new ZeroOffsetNode( new SpinnerNode( this.interceptSpinner, yIntercept, yInterceptRange, new InterceptColors(), interactiveFont, FORMAT ) );
    yInterceptNumeratorNode = new DynamicValueNode( yInterceptNumerator, FORMAT, staticFont, staticColor, true );
    yInterceptDenominatorNode = new DynamicValueNode( yInterceptDenominator, FORMAT, staticFont, staticColor, true );
    yInterceptFractionLineNode = new PPath( this.createFractionLineShape( maxSlopeSpinnerWidth ) )
    {

      {
        this.setStroke( null );
        this.setPaint( staticColor );
      }
    }
    ;
    var lineUpdater = new RichSimpleObserver()
    {

      update: function() {
        if ( !updatingControls ) {
          if ( interactiveIntercept ) {
            interactiveLine.set( Line.createSlopeIntercept( rise.get(), run.get(), yIntercept.get(), this.color ) );
          }
          else {
            var line = interactiveLine.get();
            interactiveLine.set( new Line( this.x1, this.y1, this.x1 + run.get(), this.y1 + rise.get(), this.color ) );
          }
        }
      }
    }
    ;
    lineUpdater.observe( rise, run, yIntercept );
    interactiveLine.addObserver( new VoidFunction1()
    {

      apply: function( line ) {
        assert( !interactiveIntercept || (this.x1 == 0 && MathUtil.isInteger( this.y1 )) );
        updatingControls = true;
        {
          rise.set( interactiveSlope ? this.rise : line.getSimplifiedRise() );
          run.set( interactiveSlope ? this.run : line.getSimplifiedRun() );
          if ( interactiveIntercept ) {
            yIntercept.set( this.y1 );
          }
          else {
            var fractionalIntercept = interactiveLine.get().getYIntercept();
            yInterceptNumerator.set( this.numerator );
            yInterceptDenominator.set( this.denominator );
          }
        }
        updatingControls = false;
        this.updateLayout( line, interactiveSlope, interactiveIntercept, staticFont, staticColor );
      }
    }
  )
    ;
  }

  return inherit( EquationNode, SlopeInterceptEquationNode, {

    updateLayout: function( line, interactiveSlope, interactiveIntercept, staticFont, staticColor ) {
      this.removeAllChildren();
      operatorNode.removeAllChildren();
      if ( line.undefinedSlope() && !interactiveSlope && !interactiveIntercept ) {
        this.addChild( new SlopeUndefinedNode( line, staticFont, staticColor ) );
        return;
      }
      var slope = line.getSlope();
      var zeroSlope = (slope == 0);
      var unitySlope = (Math.abs( slope ) == 1);
      var integerSlope = MathUtil.isInteger( slope );
      var positiveSlope = (slope > 0);
      var fractionalSlope = (!zeroSlope && !unitySlope && !integerSlope);
      this.addChild( yNode );
      this.addChild( equalsNode );
      yNode.setOffset( 0, 0 );
      equalsNode.setOffset( yNode.getFullBoundsReference().getMaxX() + relationalOperatorXSpacing, yNode.getYOffset() );
      if ( interactiveSlope ) {
        this.addChild( riseNode );
        this.addChild( slopeFractionLineNode );
        this.addChild( runNode );
        this.addChild( xNode );
        slopeFractionLineNode.setOffset( equalsNode.getFullBoundsReference().getMaxX() + relationalOperatorXSpacing, equalsNode.getFullBoundsReference().getCenterY() + fractionLineYFudgeFactor );
        riseNode.setOffset( slopeFractionLineNode.getFullBoundsReference().getCenterX() - (riseNode.getFullBoundsReference().getWidth() / 2), slopeFractionLineNode.getFullBoundsReference().getMinY() - riseNode.getFullBoundsReference().getHeight() - spinnersYSpacing );
        runNode.setOffset( slopeFractionLineNode.getFullBoundsReference().getCenterX() - (runNode.getFullBoundsReference().getWidth() / 2), slopeFractionLineNode.getFullBoundsReference().getMinY() + spinnersYSpacing );
        xNode.setOffset( slopeFractionLineNode.getFullBoundsReference().getMaxX() + fractionalSlopeXSpacing, yNode.getYOffset() );
      }
      else {
        var previousNode;
        var previousXOffset;
        if ( positiveSlope || zeroSlope ) {
          previousNode = equalsNode;
          previousXOffset = relationalOperatorXSpacing;
        }
        else {
          this.addChild( slopeMinusSignNode );
          slopeMinusSignNode.setOffset( equalsNode.getFullBoundsReference().getMaxX() + relationalOperatorXSpacing, equalsNode.getFullBoundsReference().getCenterY() - (slopeMinusSignNode.getFullBoundsReference().getHeight() / 2) + slopeSignYFudgeFactor + slopeSignYOffset );
          previousNode = slopeMinusSignNode;
          previousXOffset = (fractionalSlope ? fractionSignXSpacing : integerSignXSpacing);
        }
        if ( line.undefinedSlope() || fractionalSlope ) {
          this.addChild( riseNode );
          this.addChild( slopeFractionLineNode );
          this.addChild( runNode );
          this.addChild( xNode );
          var lineWidth = Math.max( riseNode.getFullBoundsReference().getWidth(), runNode.getFullBoundsReference().getWidth() );
          slopeFractionLineNode.setPathTo( this.createFractionLineShape( lineWidth ) );
          slopeFractionLineNode.setOffset( previousNode.getFullBoundsReference().getMaxX() + previousXOffset, equalsNode.getFullBoundsReference().getCenterY() + fractionLineYFudgeFactor );
          riseNode.setOffset( slopeFractionLineNode.getFullBoundsReference().getCenterX() - (riseNode.getFullBoundsReference().getWidth() / 2), slopeFractionLineNode.getFullBoundsReference().getMinY() - riseNode.getFullBoundsReference().getHeight() - ySpacing );
          runNode.setOffset( slopeFractionLineNode.getFullBoundsReference().getCenterX() - (runNode.getFullBoundsReference().getWidth() / 2), slopeFractionLineNode.getFullBoundsReference().getMinY() + ySpacing );
          xNode.setOffset( slopeFractionLineNode.getFullBoundsReference().getMaxX() + fractionalSlopeXSpacing, yNode.getYOffset() );
        }
        else if ( zeroSlope ) {
        }
        else if ( unitySlope ) {
          this.addChild( xNode );
          xNode.setOffset( previousNode.getFullBoundsReference().getMaxX() + previousXOffset, yNode.getYOffset() );
        }
        else if ( integerSlope ) {
          this.addChild( riseNode );
          this.addChild( xNode );
          riseNode.setOffset( previousNode.getFullBoundsReference().getMaxX() + previousXOffset, yNode.getYOffset() );
          xNode.setOffset( riseNode.getFullBoundsReference().getMaxX() + integerSlopeXSpacing, yNode.getYOffset() );
        }
        else {
          throw new IllegalStateException( "programming error, didn't handle some slope case" );
        }
      }
      if ( interactiveIntercept ) {
        if ( zeroSlope && !interactiveSlope ) {
          this.addChild( yInterceptNode );
          yInterceptNode.setOffset( equalsNode.getFullBoundsReference().getMaxX() + relationalOperatorXSpacing, yNode.getFullBoundsReference().getCenterY() - (yInterceptNode.getFullBoundsReference().getHeight() / 2) );
        }
        else {
          this.addChild( operatorNode );
          this.addChild( yInterceptNode );
          operatorNode.addChild( new PlusNode( operatorLineSize, staticColor ) );
          operatorNode.setOffset( xNode.getFullBoundsReference().getMaxX() + operatorXSpacing, equalsNode.getFullBoundsReference().getCenterY() - (operatorNode.getFullBoundsReference().getHeight() / 2) + operatorYFudgeFactor );
          yInterceptNode.setOffset( operatorNode.getFullBoundsReference().getMaxX() + operatorXSpacing, yNode.getFullBoundsReference().getCenterY() - (yInterceptNode.getFullBoundsReference().getHeight() / 2) );
        }
      }
      else {
        var fractionalIntercept = line.getYIntercept();
        var zeroIntercept = (fractionalIntercept.toDecimal() == 0);
        var integerIntercept = fractionalIntercept.isInteger();
        var positiveIntercept = (fractionalIntercept.toDecimal() > 0);
        if ( zeroIntercept ) {
          if ( zeroSlope && !interactiveSlope ) {
            this.addChild( yInterceptNumeratorNode );
            yInterceptNumeratorNode.setOffset( equalsNode.getFullBoundsReference().getMaxX() + relationalOperatorXSpacing, yNode.getFullBoundsReference().getCenterY() - (yInterceptNumeratorNode.getFullBoundsReference().getHeight() / 2) );
          }
          else {
          }
        }
        else if ( positiveIntercept && zeroSlope && !interactiveSlope ) {
          this.addChild( yInterceptNumeratorNode );
          yInterceptNumeratorNode.setOffset( equalsNode.getFullBoundsReference().getMaxX() + relationalOperatorXSpacing, yNode.getFullBoundsReference().getCenterY() - (yInterceptNumeratorNode.getFullBoundsReference().getHeight() / 2) );
        }
        else if ( !positiveIntercept && zeroSlope && !interactiveSlope ) {
          this.addChild( yInterceptMinusSignNode );
          this.addChild( yInterceptNumeratorNode );
          yInterceptMinusSignNode.setOffset( equalsNode.getFullBoundsReference().getMaxX() + relationalOperatorXSpacing, equalsNode.getFullBoundsReference().getCenterY() - (yInterceptMinusSignNode.getFullBoundsReference().getHeight() / 2) + operatorYFudgeFactor );
          yInterceptNumeratorNode.setOffset( yInterceptMinusSignNode.getFullBoundsReference().getMaxX() + integerSignXSpacing, yNode.getFullBoundsReference().getCenterY() - (yInterceptNumeratorNode.getFullBoundsReference().getHeight() / 2) );
        }
        else {
          this.addChild( operatorNode );
          operatorNode.addChild( positiveIntercept ? new PlusNode( operatorLineSize, staticColor ) : new MinusNode( operatorLineSize, staticColor ) );
          operatorNode.setOffset( xNode.getFullBoundsReference().getMaxX() + operatorXSpacing, equalsNode.getFullBoundsReference().getCenterY() - (operatorNode.getFullBoundsReference().getHeight() / 2) + operatorYFudgeFactor );
          if ( integerIntercept ) {
            this.addChild( yInterceptNumeratorNode );
            yInterceptNumeratorNode.setOffset( operatorNode.getFullBoundsReference().getMaxX() + operatorXSpacing, yNode.getFullBoundsReference().getCenterY() - (yInterceptNumeratorNode.getFullBoundsReference().getHeight() / 2) );
          }
          else {
            this.addChild( yInterceptNumeratorNode );
            this.addChild( yInterceptFractionLineNode );
            this.addChild( yInterceptDenominatorNode );
            var lineWidth = Math.max( yInterceptNumeratorNode.getFullBoundsReference().getWidth(), yInterceptDenominatorNode.getFullBoundsReference().getWidth() );
            yInterceptFractionLineNode.setPathTo( this.createFractionLineShape( lineWidth ) );
            yInterceptFractionLineNode.setOffset( operatorNode.getFullBoundsReference().getMaxX() + operatorXSpacing, equalsNode.getFullBoundsReference().getCenterY() + fractionLineYFudgeFactor );
            yInterceptNumeratorNode.setOffset( yInterceptFractionLineNode.getFullBoundsReference().getCenterX() - (yInterceptNumeratorNode.getFullBoundsReference().getWidth() / 2), yInterceptFractionLineNode.getFullBoundsReference().getMinY() - yInterceptNumeratorNode.getFullBoundsReference().getHeight() - ySpacing );
            yInterceptDenominatorNode.setOffset( yInterceptFractionLineNode.getFullBoundsReference().getCenterX() - (yInterceptDenominatorNode.getFullBoundsReference().getWidth() / 2), yInterceptFractionLineNode.getFullBoundsReference().getMaxY() + ySpacing );
          }
        }
      }
      if ( line.undefinedSlope() ) {
        var undefinedSlopeIndicator = new UndefinedSlopeIndicator( this.getFullBoundsReference().getWidth(), this.getFullBoundsReference().getHeight() );
        undefinedSlopeIndicator.setOffset( 0, slopeFractionLineNode.getFullBoundsReference().getCenterY() - (undefinedSlopeIndicator.getFullBoundsReference().getHeight() / 2) + undefinedSlopeYFudgeFactor );
        this.addChild( undefinedSlopeIndicator );
      }
    },


    createGeneralFormNode: function() {
      var html = MessageFormat.format( "{0} = {1}{2} + {3}", this.SYMBOL_Y, this.SYMBOL_SLOPE, this.SYMBOL_X, this.SYMBOL_INTERCEPT );
      return new HTMLNode( html, this.INTERACTIVE_LINE, new PhetFont( this.BOLD, 18 ) );
    },


  } );

} );

